var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (Object.prototype.hasOwnProperty.call(b, p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        if (typeof b !== "function" && b !== null)
            throw new TypeError("Class extends value " + String(b) + " is not a constructor or null");
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
// common
import { Loggable } from "../../../common/log";
// entities
import { LocationConnectionPath } from "./connection-path";
// babylon
import * as BABYLON from 'babylonjs';
/* == CONNECTION MODEL == */
var TradeConnection = /** @class */ (function (_super) {
    __extends(TradeConnection, _super);
    function TradeConnection(source, target, findPath) {
        var _this = _super.call(this, "CityConnection") || this;
        _this.level = 2;
        // trade related properties
        _this.doneTrades = 0;
        _this.tradedCoins = 0;
        _this.tilePath = null;
        _this.source = source;
        _this.target = target;
        // find distances
        _this.tileDistance = _this._findTileDistance();
        // find path if required
        _this.path = new LocationConnectionPath(_this, source, target);
        return _this;
    }
    TradeConnection.prototype._findTileDistance = function () {
        var distX = Math.abs(this.source.pos.x - this.target.pos.x);
        var distZ = Math.abs(this.source.pos.z - this.target.pos.z);
        return distX + distZ;
    };
    Object.defineProperty(TradeConnection.prototype, "worldDistance", {
        get: function () {
            var sourceMesh = this.source.mesh;
            var targetMesh = this.target.mesh;
            if (!(sourceMesh && targetMesh)) {
                this.warn("cant find distance between city meshes ".concat(sourceMesh, " and ").concat(targetMesh));
                return -1;
            }
            var worldDist = BABYLON.Vector3.Distance(sourceMesh.position, targetMesh.position);
            return Math.round(worldDist);
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(TradeConnection.prototype, "locations", {
        get: function () {
            return [this.source, this.target];
        },
        enumerable: false,
        configurable: true
    });
    TradeConnection.prototype.getDistance = function (distanceType) {
        if (distanceType == DISTANCE_TYPE_TILES) {
            return this.tileDistance;
        }
        else {
            return this.worldDistance;
        }
    };
    Object.defineProperty(TradeConnection.prototype, "mirrored", {
        get: function () {
            if (globalThis.CONN) {
                return globalThis.CONN.getMirrored(this);
            }
            return;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(TradeConnection.prototype, "averageWealth", {
        get: function () {
            return (this.source.money + this.target.money) / 2;
        },
        enumerable: false,
        configurable: true
    });
    TradeConnection.prototype.addToTradeHistory = function (marketTrade, updateMirrored) {
        this.doneTrades += 1;
        this.tradedCoins += marketTrade.profit;
        if (updateMirrored && updateMirrored == true) {
            this.mirrored.doneTrades += 1;
            this.mirrored.tradedCoins += marketTrade.profit;
        }
    };
    TradeConnection.prototype.hasTarget = function (target) {
        if (this.target.key === target.key) {
            return true;
        }
        return false;
    };
    Object.defineProperty(TradeConnection, "MAX_DISTANCE", {
        /* CONSTANTS */
        get: function () { return 12; },
        enumerable: false,
        configurable: true
    });
    return TradeConnection;
}(Loggable));
export { TradeConnection };
/* == CONNECTION CONSTANTS == */
export var DISTANCE_TYPE_TILES = 0;
export var DISTANCE_TYPE_WORLD = 1;
//# sourceMappingURL=connection.js.map