var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (Object.prototype.hasOwnProperty.call(b, p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        if (typeof b !== "function" && b !== null)
            throw new TypeError("Class extends value " + String(b) + " is not a constructor or null");
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
// common
import { Loggable } from "../../../common/log";
import { ResourceTypes } from "./resource-types";
/* == STORED RESOURCE == */
var StoredResource = /** @class */ (function (_super) {
    __extends(StoredResource, _super);
    function StoredResource(resId, amount, price, profit) {
        var _this = _super.call(this, "StoredRes") || this;
        // optional
        _this.price = null;
        _this.profit = null;
        _this.resId = resId;
        _this.amount = amount;
        if (price) {
            _this.price = price;
        }
        if (profit) {
            _this.profit = profit;
        }
        return _this;
    }
    Object.defineProperty(StoredResource.prototype, "type", {
        get: function () {
            if (!globalThis.RESOURCE_TYPES) {
                this.log("cant get resource type; resource types not loaded");
                return;
            }
            else if (!globalThis.TRADE) {
                this.log("cant get resource type; TradeManager not loaded");
                return;
            }
            return globalThis.TRADE.getRes(this.resId);
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(StoredResource.prototype, "tradeRes", {
        get: function () { return this.type; },
        enumerable: false,
        configurable: true
    });
    StoredResource.prototype.toString = function () {
        return "<StoredRes ".concat(this.amount, "x").concat(this.resId, " price=").concat(this.price, ">");
    };
    StoredResource.fromString = function (storedResStr) {
        var splittedStr = storedResStr.split("x");
        var amount = parseFloat(splittedStr[0]);
        var resId = splittedStr[1];
        return new StoredResource(resId, amount);
    };
    return StoredResource;
}(Loggable));
export { StoredResource };
/* == STORAGE MODEL == */
var ResourceStorage = /** @class */ (function (_super) {
    __extends(ResourceStorage, _super);
    function ResourceStorage(maxSpace, parent, justFor) {
        var _this = _super.call(this, "ResourceStorage") || this;
        _this.spaceTaken = 0;
        _this.resIds = [];
        _this._resAmount = {};
        _this._resPrice = {};
        _this.maxSpace = maxSpace;
        if (parent) {
            _this.parent = parent;
        }
        if (justFor && justFor.length > 0) {
            _this.initializeForTypes(justFor);
        }
        _this.initializeEmpty();
        return _this;
    }
    /* add "amount" of resources */
    ResourceStorage.prototype.add = function (resId, amount) {
        if (this.spaceTaken < this.maxSpace) {
            this._resAmount[resId] += amount;
            this.spaceTaken += amount;
            // this.log(`added ${amount} of ${resId} to storage`);
            return true;
        }
        // this.error(`cant add ${amount} of ${resId} to storage; is already full`);
        return false;
    };
    /* remove "amount" resources without checking whether there are enough left or changing prices */
    ResourceStorage.prototype.remove = function (resId, amount, force) {
        var storedAmount = this._resAmount[resId];
        if (storedAmount >= amount) {
            this._resAmount[resId] -= amount;
            // this.log(`removed ${amount} of ${resId} from storage`);
            return true;
        }
        else {
            // this.error(`cant remove ${amount} of ${resId} from storage; just ${storedAmount} left`);
            return false;
        }
    };
    /* returns how many units of a resource is stored here */
    ResourceStorage.prototype.amountOf = function (resId) {
        return this._resAmount[resId];
    };
    ResourceStorage.prototype.initializeEmpty = function () {
        var allResIds = new ResourceTypes()._resourceIds;
        for (var _i = 0, allResIds_1 = allResIds; _i < allResIds_1.length; _i++) {
            var resId = allResIds_1[_i];
            this.resIds.push(resId);
            this._resAmount[resId] = 0;
            var defaultPrice = globalThis.RESOURCE_TYPES.get(resId).defaultPrice;
            this._resPrice[resId] = defaultPrice;
        }
    };
    ResourceStorage.prototype.initializeForTypes = function (forIds) {
        for (var _i = 0, forIds_1 = forIds; _i < forIds_1.length; _i++) {
            var resId = forIds_1[_i];
            this.resIds.push(resId);
            this._resAmount[resId] = 0;
            var defaultPrice = globalThis.RESOURCE_TYPES.get(resId).defaultPrice;
            this._resPrice[resId] = defaultPrice;
        }
    };
    ResourceStorage.prototype.getStoredResource = function (resId) {
        var amount = this.amountOf(resId);
        var stored = new StoredResource(resId, amount);
        return stored;
    };
    return ResourceStorage;
}(Loggable));
export { ResourceStorage };
//# sourceMappingURL=resource-storage.js.map