var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (Object.prototype.hasOwnProperty.call(b, p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        if (typeof b !== "function" && b !== null)
            throw new TypeError("Class extends value " + String(b) + " is not a constructor or null");
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
// trade
import { ResourceStorage } from "./resource-storage";
// entities
import { City } from "../city/city";
import { MarketTrade } from "./market-trade";
/* == MARKET MODEL == */
var TradeNode = /** @class */ (function (_super) {
    __extends(TradeNode, _super);
    function TradeNode(maxSpace, fixedPrices, parent, justFor) {
        var _this = _super.call(this, maxSpace, parent, justFor) || this;
        /* markets have no maximal storage amount */
        /* justFor: dont create storage for all resources but for specified resource ids (in array) */
        _this.fixedPrices = false;
        _this.linked = [];
        _this.parentLocation = null;
        _this.history = [];
        if (parent instanceof City) {
            _this.parentLocation = parent;
        }
        _this.fixedPrices = fixedPrices;
        return _this;
    }
    TradeNode.prototype.asStoredResources = function () {
        var storedResources = [];
        for (var _i = 0, _a = this.resIds; _i < _a.length; _i++) {
            var resId = _a[_i];
            var stored = this.getStoredResource(resId);
            storedResources.push(stored);
        }
        return storedResources;
    };
    TradeNode.prototype.add = function (resId, amount, price) {
        var success = _super.prototype.add.call(this, resId, amount);
        if (success) {
            if (!this.fixedPrices) {
                this.reducePrice(resId, amount);
            }
            else if (price) {
                this._resPrice = price;
            }
            return true;
        }
        return false;
    };
    TradeNode.prototype.remove = function (resId, amount, force) {
        var success = _super.prototype.remove.call(this, resId, amount, force);
        if (!success) {
            this.increasePrice(resId, amount);
        }
        return success;
    };
    /* TRADE NODE GETTERS */
    TradeNode.prototype.tradeIsExport = function (marketTrade) {
        return this.parentLocation.equals(marketTrade.source);
    };
    TradeNode.prototype.priceOf = function (resId) {
        return Math.round(this._resPrice[resId]);
    };
    /* returns the difference to a resource (ids) default/start price */
    TradeNode.prototype.defaultPriceDiffOf = function (resId) {
        var tradeRes = globalThis.RESOURCE_TYPES.get(resId);
        // replace with special resource when loaded
        if (globalThis.TRADE && globalThis.TRADE.hasSpecialResLoaded) {
            tradeRes = globalThis.TRADE.getRes(resId);
        }
        return tradeRes.defaultPrice - this.priceOf(resId);
    };
    /* returns whether a resource has minAmount left in storage */
    TradeNode.prototype.hasEnough = function (resId, minAmount) {
        return this.amountOf(resId) >= minAmount;
    };
    /* returns whether the requirements to produce resId in given quantity are met */
    TradeNode.prototype.hasRequirementsFor = function (resId, toProduce) {
        var resource = globalThis.RESOURCE_TYPES.get(resId);
        if (globalThis.TRADE && globalThis.TRADE.hasSpecialResLoaded) {
            resource = globalThis.TRADE.getRes(resId);
        }
        if (resource.hasRequirement) {
            for (var _i = 0, _a = resource.requires; _i < _a.length; _i++) {
                var requiredResId = _a[_i];
                if (!this.hasEnough(requiredResId, toProduce)) {
                    // make more expensive
                    this.increasePrice(resId, toProduce);
                    return false;
                }
            }
        }
        return true;
    };
    TradeNode.prototype.increasePrice = function (resId, unmetAmount) {
        var increaseAmount = Math.round(this.priceOf(resId) / 20);
        this._resPrice[resId] += increaseAmount;
    };
    TradeNode.prototype.reducePrice = function (resId, unmetAmount) {
        var decreaseAmount = Math.round(this.priceOf(resId) / 20);
        this._resPrice[resId] -= decreaseAmount;
    };
    /* TRADE NOTE GETTER METHODS */
    /* returns an array of 3 resources that this storage has the most of */
    TradeNode.prototype.getMostStored = function () {
        var sortedByAmount = this.asStoredResources().sort(function (stored1, stored2) {
            if (stored1.amount < stored2.amount) {
                return 1;
            }
            if (stored1.amount > stored2.amount) {
                return -1;
            }
            return 0;
        });
        //  console.log(sortedByAmount);
        return [sortedByAmount[0], sortedByAmount[1], sortedByAmount[2]];
    };
    TradeNode.prototype.compareResourceProfitWith = function (resId, otherMarket) {
        var otherMarketStoredRes = otherMarket.getStoredResource(resId);
        return this.priceOf(resId) - otherMarketStoredRes.price;
    };
    TradeNode.prototype.getProfitFor = function (storedRes) {
        if (!storedRes || !storedRes.price) {
            this.error("cant get profit for resource ".concat(storedRes.resId, "; no price given"));
            return;
        }
        var storedPrice = storedRes.price;
        var marketPrice = this.priceOf(storedRes.resId);
        return storedPrice - marketPrice;
    };
    TradeNode.prototype.getTradeProfitsBetween = function (otherMarket) {
        var profits = [];
        for (var _i = 0, _a = globalThis.RESOURCE_TYPES.getIds(); _i < _a.length; _i++) {
            var resId = _a[_i];
            if (this.amountOf(resId) > 0 || otherMarket.amountOf(resId) > 0) {
                var otherRes = otherMarket.getStoredResource(resId);
                var profit = this.getProfitFor(otherRes);
                profits.push(new MarketTrade(null, resId, profit));
            }
        }
        return profits;
    };
    TradeNode.prototype.getBestToBuy = function (amount) {
        var storage = this;
        // sort StoredResources by price difference
        var sortedByPriceDiff = this.asStoredResources().sort(function (stored1, stored2) {
            var priceDiff1 = storage.defaultPriceDiffOf(stored1.resId);
            var priceDiff2 = storage.defaultPriceDiffOf(stored2.resId);
            if (priceDiff1 < priceDiff2) {
                return 1;
            }
            if (priceDiff1 > priceDiff2) {
                return -1;
            }
            return 0;
        });
        if (amount) {
            return sortedByPriceDiff.slice(0, amount);
        }
        else {
            return sortedByPriceDiff;
        }
    };
    TradeNode.prototype.getStoredResource = function (resId) {
        var stored = _super.prototype.getStoredResource.call(this, resId);
        stored.price = this.priceOf(resId);
        return stored;
    };
    TradeNode.prototype.getEmpty = function () {
        var maxAmount = 3;
        var emptyResources = [];
        for (var _i = 0, _a = this.asStoredResources(); _i < _a.length; _i++) {
            var storedRes = _a[_i];
            if (storedRes.amount <= 0 && emptyResources.length < maxAmount) {
                emptyResources.push(storedRes);
            }
        }
        return emptyResources;
    };
    return TradeNode;
}(ResourceStorage));
export { TradeNode };
//# sourceMappingURL=trade-node.js.map