var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (Object.prototype.hasOwnProperty.call(b, p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        if (typeof b !== "function" && b !== null)
            throw new TypeError("Class extends value " + String(b) + " is not a constructor or null");
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
import { BaseAbility } from "../../../../common/ability";
import { Loggable } from "../../../../common/log";
import { Properties } from "../../../../common/properties";
/* WEAPON MODEL */
var UnitWeapon = /** @class */ (function () {
    function UnitWeapon(id) {
        this.properties = new Properties();
        this.abilities = [];
    }
    Object.defineProperty(UnitWeapon.prototype, "name", {
        /* PROPERTY GETTERS */
        get: function () { return this.properties.getStr("name"); },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(UnitWeapon.prototype, "baseAtk", {
        get: function () { return this.properties.getNum("baseAtk"); },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(UnitWeapon.prototype, "criticalChance", {
        get: function () { return this.properties.getNum("criticalChance"); },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(UnitWeapon.prototype, "distance", {
        get: function () { return this.properties.getNum("distance"); },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(UnitWeapon.prototype, "weaponCat", {
        get: function () { return this.properties.getNum("weaponCat"); },
        enumerable: false,
        configurable: true
    });
    UnitWeapon.prototype.atkModVsArmourCat = function (armourCat) {
        // magic attack
        if (this.weaponCat == WEAPON_CAT_MAGIC) {
            return 1;
        }
        // pierce vs heavy
        var pierceVsHeavy = this.weaponCat == WEAPON_CAT_PIERCE && armourCat == ARMOUR_CAT_HEAVY;
        if (pierceVsHeavy) {
            return 2;
        }
        // bash vs light
        var bashVsLight = this.weaponCat == WEAPON_CAT_BASH && armourCat == ARMOUR_CAT_LIGHT;
        if (bashVsLight) {
            return 2;
        }
        // bash vs heavy 
        var bashVsHeavy = this.weaponCat == WEAPON_CAT_BASH && armourCat == ARMOUR_CAT_HEAVY;
        if (bashVsHeavy) {
            return .6;
        }
        // fire vs vehicles
        var fireVsVehicle = this.weaponCat == WEAPON_CAT_FIRE && armourCat == ARMOUR_CAT_VEHICLE;
        if (fireVsVehicle) {
            return 3;
        }
        return 1;
    };
    UnitWeapon.prototype.damageVsArmour = function (armourId) {
        var baseDmg = this.baseAtk;
        var enemyArmourCat = new UnitArmour(armourId).armourCat;
        var dmgMod = this.atkModVsArmourCat(enemyArmourCat);
        var totalDmg = baseDmg * dmgMod;
        return totalDmg;
    };
    UnitWeapon.FromJson = function (jsonObj) {
        var weaponObj = new UnitWeapon(jsonObj["id"]);
        weaponObj.properties.addString("name", jsonObj["name"]);
        weaponObj.properties.addFloat("baseAtk", jsonObj["baseAtk"]);
        weaponObj.properties.addFloat("criticalChance", jsonObj["criticalChance"]);
        weaponObj.properties.addInteger("distance", jsonObj["distance"]);
        weaponObj.properties.addInteger("dmgType", jsonObj["dmgType"]);
        // add weapon abilities
        for (var _i = 0, _a = jsonObj["abilities"]; _i < _a.length; _i++) {
            var abilityStr = _a[_i];
            var ability = BaseAbility.FromString(abilityStr);
            weaponObj.abilities.push(ability);
        }
        return weaponObj;
    };
    return UnitWeapon;
}());
export { UnitWeapon };
var UnitArmour = /** @class */ (function () {
    function UnitArmour(id) {
        this.properties = new Properties();
        this.id = id;
    }
    Object.defineProperty(UnitArmour.prototype, "name", {
        /* GETTERS & SETTERS */
        get: function () { return this.properties.getStr("name"); },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(UnitArmour.prototype, "category", {
        get: function () { return this.properties.getNum("category"); },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(UnitArmour.prototype, "armourCat", {
        get: function () { return this.category; },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(UnitArmour.prototype, "extraDefense", {
        get: function () { return this.properties.getNum("extraDefense"); },
        enumerable: false,
        configurable: true
    });
    UnitArmour.FromJson = function (jsonObj) {
        var armorObj = new UnitArmour(jsonObj["id"]);
        armorObj.properties.addString("name", jsonObj["name"]);
        armorObj.properties.addFloat("category", jsonObj["category"]);
        armorObj.properties.addFloat("extraDefense", jsonObj["extraDefense"]);
        return armorObj;
    };
    return UnitArmour;
}());
export { UnitArmour };
/* WEAPON & ARMOUR SET */
var UnitGear = /** @class */ (function (_super) {
    __extends(UnitGear, _super);
    function UnitGear(str, autoInitialize) {
        var _this = _super.call(this, "UnitGear") || this;
        _this.hasLoaded = false;
        _this.str = str;
        var weaponArmourIds = str.split(" ");
        _this._weaponId = weaponArmourIds[0];
        _this._armourId = weaponArmourIds[1];
        if (autoInitialize && autoInitialize == true) {
            _this.fillGearTypeObjects();
        }
        return _this;
    }
    UnitGear.prototype.fillGearTypeObjects = function () {
        if (!globalThis.GEAR) {
            this.warn("cant fill gear type objects; UnitGearManager not found");
            return;
        }
        // this.weapon = globalThis.GEAR.getWeaponById(this._weaponId);
        // this.armour = globalThis.GEAR.getArmorById(this._armourId);
        this.hasLoaded = true;
    };
    Object.defineProperty(UnitGear.prototype, "weapon", {
        /* GETTERS */
        get: function () {
            if (!globalThis.GEAR) {
                this.error("cant load weapon \"".concat(this._weaponId, "\"; UnitGearManager not found"));
                return;
            }
            return globalThis.GEAR.getWeaponById(this._weaponId);
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(UnitGear.prototype, "armour", {
        get: function () {
            if (!globalThis.GEAR) {
                this.error("cant load armor \"".concat(this._armourId, "\"; UnitGearManager not found"));
                return;
            }
            return globalThis.GEAR.getArmorById(this._armourId);
        },
        enumerable: false,
        configurable: true
    });
    return UnitGear;
}(Loggable));
export { UnitGear };
/* == GEAR CONSTANTS */
// weapons
export var WEAPON_CRITICAL_DMG_MOD = 2;
// natural
export var WEAPON_BITE = "bite";
export var WEAPON_CLAW = "claw";
export var WEAPON_CRUSH = "crush";
export var WEAPON_IDS = ["dagger", "spear", "sword", "bow", "lance", "longsword", "crossbow", "javelin", "mace", "bomb", "bite", "claw", "crush"];
// damage categories
export var WEAPON_CAT_CUT = 0;
export var WEAPON_CAT_PIERCE = 1;
export var WEAPON_CAT_BASH = 2;
export var WEAPON_CAT_FIRE = 3;
export var WEAPON_CAT_MAGIC = 4;
// armour
export var ARMOUR_CLOTH = "cloth";
export var ARMOUR_LEATHER = "leather";
export var ARMOUR_BRONZE = "bronze";
export var ARMOUR_IRON = "iron";
export var ARMOUR_PLATED = "plated";
export var ARMOUR_VEHICLE = "vehicle";
export var ARMOUR_VEHICLE_HEAVY = "heavy_vehicle";
// armour categories
export var ARMOUR_CAT_LIGHT = 1;
export var ARMOUR_CAT_MEDIUM = 2;
export var ARMOUR_CAT_HEAVY = 3;
export var ARMOUR_CAT_VEHICLE = 8;
//# sourceMappingURL=gear.js.map