var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (Object.prototype.hasOwnProperty.call(b, p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        if (typeof b !== "function" && b !== null)
            throw new TypeError("Class extends value " + String(b) + " is not a constructor or null");
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
// gui
import { AbstractGuiElement } from "./element";
import { Switcher } from "../elements/switcher";
import * as GUI_CONST from "./constants";
import { SingleDataRow } from "../elements/data-row";
// babylon
import * as GUI from 'babylonjs-gui';
import { DawnbringerPalette } from "../../../common/colors";
import { SectionHeader } from "../elements/section-header";
import { GuiTools } from "./gui-tools";
var AbstractGuiContainer = /** @class */ (function (_super) {
    __extends(AbstractGuiContainer, _super);
    function AbstractGuiContainer(id) {
        var _this = _super.call(this, id) || this;
        // layout properties
        _this.children = [];
        _this.childrenIds = [];
        _this.maxSize = null;
        _this.dividers = 0;
        // defaults
        _this.forceEmpty = false;
        return _this;
    }
    AbstractGuiContainer.prototype.add = function (elem) {
        this.log("adding ".concat(elem));
        if (this.forceEmpty) {
            this.error("cant add ".concat(elem, "; is forced empty"));
        }
        this.childrenIds.push(elem.id);
        this.children.push(elem);
        this.onElementAdded(elem);
    };
    AbstractGuiContainer.prototype.addMultiple = function (elements) {
        for (var _i = 0, elements_1 = elements; _i < elements_1.length; _i++) {
            var elem = elements_1[_i];
            this.add(elem);
        }
    };
    AbstractGuiContainer.prototype.nextDividerNum = function () {
        this.dividers += 1;
        return this.dividers;
    };
    /* CONTENT */
    AbstractGuiContainer.prototype.addCustomContent = function () {
        // overwrite this method to add any BABYLON Controls as children
        this.warn("not adding any content; have you overwritten addCustomContent()?");
    };
    AbstractGuiContainer.prototype.updateCustomContent = function () {
        // overwrite this mission to update content: per example change a label's text
    };
    AbstractGuiContainer.prototype.clear = function () {
        for (var _i = 0, _a = this.children; _i < _a.length; _i++) {
            var child = _a[_i];
            child.dispose();
        }
        this.childrenIds = [];
        this.children = [];
        this.onCleared();
    };
    AbstractGuiContainer.prototype.addControl = function (control) {
        return this.babylon.addControl(control);
    };
    /* ON CHANGE */
    AbstractGuiContainer.prototype.onElementAdded = function (elem) {
        elem.parent = this;
        this.log("added element \"".concat(elem.id, "\""));
    };
    AbstractGuiContainer.prototype.onElementClicked = function (elemId) {
        this.log("click on element \"".concat(elemId, "\""));
        return false;
    };
    AbstractGuiContainer.prototype.onCleared = function () {
        this.log("cleared");
    };
    return AbstractGuiContainer;
}(AbstractGuiElement));
export { AbstractGuiContainer };
var FlexGuiLayout = /** @class */ (function (_super) {
    __extends(FlexGuiLayout, _super);
    function FlexGuiLayout(id, dir, title, maxSize, parent) {
        var _this = _super.call(this, id) || this;
        _this.title = null;
        _this.dir = dir;
        // optional properties
        if (title) {
            _this.title = title;
        }
        if (maxSize) {
            _this.maxSize = maxSize;
        }
        // build!
        _this.init();
        return _this;
    }
    FlexGuiLayout.prototype.init = function () { };
    FlexGuiLayout.prototype.build = function (isWide) {
        // create empty StackPanel
        this.babylon = this.buildContainer(isWide);
        // add header label with title text
        this.addDialogHeader();
        // add method to allow custom layout content by overwriting it
        this.addCustomContent();
        this.adjustOutterSize();
    };
    FlexGuiLayout.prototype.adjustOutterSize = function () {
        this.outter.widthInPixels = this.babylon.widthInPixels + 40;
        this.outter.height = "95%";
        this.outter.horizontalBar.parent.isVisible = false;
        this.outter.verticalBar.parent["thickness"] = 0;
    };
    FlexGuiLayout.prototype.addControl = function (control, customSize) {
        var container = this.babylon;
        if (!container) {
            this.error("cant add \"".concat(control.name, "\"; container StackPanel ").concat(this.babylon, " not initialized"));
            return;
        }
        // custom size?
        if (!customSize) {
            control.height = GUI_CONST.DEFAULT_ELEMENT_HEIGHT;
        }
        return container.addControl(control);
    };
    FlexGuiLayout.prototype.buildContainer = function (isWide) {
        // build outter
        this.outter = new GUI.ScrollViewer(this.id + "-outter");
        this.outter.scrollBackground = "#00000000";
        this.outter.thickness = 0;
        this.outter.height = "90%";
        if (isWide) {
            this.outter.width = GUI_CONST.DIALOG_OUTTER_WIDTH;
        }
        else {
            this.outter.widthInPixels = GUI_CONST.DEFAULT_INNER_WIDTH_PX + 40;
        }
        // build container
        var container = new GUI.StackPanel(this.id);
        if (isWide) {
            container.widthInPixels = GUI_CONST.DIALOG_OUTTER_WIDTH_PX;
        }
        else {
            container.width = GUI_CONST.DEFAULT_INNER_WIDTH;
        }
        // add container to outter & outte to gui root
        this.outter.addControl(container);
        globalThis.GUI.advTexture.addControl(this.outter);
        return container;
    };
    /* == ADD STANDARD ELEMENTS == */
    FlexGuiLayout.prototype.addDialogHeader = function () {
        if (!this.title) {
            return;
        }
        // create container
        var outter = new GUI.Rectangle("".concat(this.id, "-header"));
        outter.width = GUI_CONST.DEFAULT_WIDTH;
        outter.height = GUI_CONST.DEFAULT_ELEMENT_HEIGHT;
        outter.thickness = 0;
        // background image
        var bgImage = new GUI.Image("".concat(this.id, "-img"), GUI_CONST.BG_HEADER);
        bgImage.height = GUI_CONST.DEFAULT_ELEMENT_HEIGHT;
        // actual label
        var label = new GUI.TextBlock(this.id, this.title);
        label.height = GUI_CONST.DEFAULT_ELEMENT_HEIGHT;
        label.fontSize = GUI_CONST.TEXTSIZE_BIGGEST;
        label.fontFamily = GUI_CONST.FONT_HIGHLIGHTED;
        // color: dawnbringer golden fizz
        var hexColor = BABYLON.Color3.FromArray(DawnbringerPalette.GOLDENFIZZ).toHexString();
        label.color = hexColor;
        // add to container
        outter.addControl(bgImage);
        outter.addControl(label);
        this.addControl(outter);
        this.addNewDivider(true);
    };
    FlexGuiLayout.prototype.addNewLabel = function (labelId, text, clickable, smallHeight, noBackground, setParent) {
        var parent = this.babylon;
        if (setParent) {
            parent = setParent;
        }
        var labelElem = GuiTools.addNewGuiLabel(parent, labelId, text, clickable, smallHeight, noBackground);
        // return actual label
        return labelElem.labelTextBlock;
    };
    FlexGuiLayout.prototype.addNewDivider = function (empty, setParent) {
        var parent = this.babylon;
        if (setParent) {
            parent = setParent;
        }
        var dividerNum = this.nextDividerNum();
        var dividerText = "";
        if (!empty) {
            dividerText = "--------";
        }
        var dividerLabel = this.addNewLabel("divider-".concat(dividerNum), dividerText, false, true, false, parent);
        dividerLabel.width = GUI_CONST.DEFAULT_INNER_WIDTH;
        dividerLabel.textVerticalAlignment = GUI_CONST.ALIGN_V_CENTER;
        return dividerLabel;
    };
    FlexGuiLayout.prototype.addNewSwitcher = function (elemId, options) {
        var switcher = new Switcher(elemId, options);
        this.addControl(switcher);
        this.addNewDivider(true);
    };
    FlexGuiLayout.prototype.addNewSectionHeader = function (headerName) {
        var sectionHeader = new SectionHeader(headerName);
        this.addControl(sectionHeader, true);
    };
    FlexGuiLayout.prototype.addNewDataRow = function (key, value, inlineIcon, clickable, overwriteElemId, overwriteParent) {
        var layout = this;
        var dataRow = new SingleDataRow(key, value, inlineIcon, overwriteElemId, clickable);
        if (clickable) {
            dataRow.onPointerDownObservable.add(function () {
                var elemId = dataRow.name;
                if (layout) {
                    layout.onElementClicked(elemId);
                }
            });
        }
        if (!overwriteParent) {
            this.addControl(dataRow, true);
        }
        else {
            overwriteParent.addControl(dataRow);
        }
        return dataRow;
    };
    FlexGuiLayout.prototype.onElementClicked = function (elemId) {
        if (!this) {
            return false;
        }
        if (!elemId) {
            return false;
        }
        this.log("non button clicked: \"".concat(elemId, "\""));
        return true;
    };
    return FlexGuiLayout;
}(AbstractGuiContainer));
export { FlexGuiLayout };
//# sourceMappingURL=layouts.js.map