var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (Object.prototype.hasOwnProperty.call(b, p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        if (typeof b !== "function" && b !== null)
            throw new TypeError("Class extends value " + String(b) + " is not a constructor or null");
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
// common
import { DawnbringerPalette } from "../../../common/colors";
import * as EV from "../../game-events";
// gui basics
import * as GUI_CONST from '../common/constants';
import { EmptyDialog } from "./common/base-dialog";
// gui dialogs
import { ResourceListDialog } from "./reslist-dialog";
import { CityConnectionsDialog } from "./connections-dialog";
import { CityPopulationDialog } from "./city-population-dialog";
import { CityBuildingsDialog } from "./city-buildings-dialog";
import { ArmiesDialog } from "./armies-dialog";
import { RecruitmentDialog } from "./recruitment-dialog";
// babylon
import * as GUI from 'babylonjs-gui';
/* == CITY DIALOG == */
/* Main dialog that is opened on city click (or possibly entering);
   Shows name, owner, money plus clickable datarows for more information to population etc */
var CityDialog = /** @class */ (function (_super) {
    __extends(CityDialog, _super);
    function CityDialog(city) {
        var _this = _super.call(this, DIAG_CITY, false, city.name, null, false, false, city.ownerEmpire) || this;
        _this.city = city;
        _this.onlyView = !_this.playerIsInCity;
        return _this;
    }
    Object.defineProperty(CityDialog.prototype, "playerIsInCity", {
        get: function () {
            var playerPos = globalThis.PLAYER.pos;
            return playerPos && this.city.pos.equals(playerPos);
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(CityDialog.prototype, "cityIsPlayerOwned", {
        get: function () {
            var playerFamilyKey = globalThis.PLAYER.family.key;
            return this.city.ownerFamily && this.city.ownerFamily.key == playerFamilyKey;
        },
        enumerable: false,
        configurable: true
    });
    CityDialog.prototype.build = function () {
        _super.prototype.build.call(this);
    };
    CityDialog.prototype.addCustomContent = function () {
        // connections
        var connectionsText = "".concat(this.city.connections.length);
        this.addNewDataRow("Connections", connectionsText, "connection", true, DIAG_CITY_LBL_CONNECTIONS);
        // properties
        this.addCityPropertiesBar();
        // owner
        this.addNewDataRow("Owner", this.city.ownerName, "empire", false);
        // leader
        var leaderName = "";
        if (this.city.leader && this.city.leader.fullName) {
            leaderName = this.city.leader.fullName;
        }
        this.addNewDataRow("Leader", leaderName, "trader", false);
        // money
        var moneyText = "" + Math.round(this.city.money);
        this.addNewDataRow("Money", moneyText, "money", false);
        // pops
        var growthIncreaseStr = " (Grw.: +".concat(this.lastGrowthIncrease.toFixed(1), ")");
        var popsText = Math.round(this.city.population.totalPops) + growthIncreaseStr;
        this.addNewDataRow("Population", popsText, "growth", true, DIAG_CITY_LBL_POPS);
        // buildings
        var buildingsText = "" + this.city.buildings.length;
        this.addNewDataRow("Buildings", buildingsText, "building", true, DIAG_CITY_LBL_BUILDINGS);
        // milita
        var militiaText = this.getMiltiaUnitTypes();
        this.addNewDataRow("Militia", militiaText, "might", true, DIAG_CITY_LBL_MILITIA);
        this.addNewDivider(true);
        // player is currently in this city?
        if (this.playerIsInCity) {
            // res info
            this.addResourceInformation();
            // add trade & recruitment button
            this.addNewButton(DIAG_CITY_BTN_TRADE, "Trade");
            this.addNewButton(DIAG_CITY_BTN_RECRUIT, "Recruit");
        }
        // city is owned by this player's family?
        if (this.cityIsPlayerOwned) {
            // plus city command options if city is player owned
            this.addNewButton(DIAG_CITY_BTN_COMMAND, "Command");
        }
    };
    CityDialog.prototype.addCityPropertiesBar = function () {
    };
    Object.defineProperty(CityDialog.prototype, "resHappinessStr", {
        get: function () {
            var resHappinessPercent = Math.round(this.city.population.lastConsumptionSuccess * 100);
            return "".concat(resHappinessPercent, "%");
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(CityDialog.prototype, "lastGrowthIncrease", {
        get: function () { return this.city.population.totalGrowthIncrease; },
        enumerable: false,
        configurable: true
    });
    CityDialog.prototype.getMiltiaUnitTypes = function () {
        if (!this.city.militia) {
            return "empty";
        }
        var unitNames = [];
        for (var _i = 0, _a = this.city.militia.units; _i < _a.length; _i++) {
            var unit = _a[_i];
            unitNames.push(unit.type.id);
        }
        return unitNames.join(", ");
    };
    CityDialog.prototype.addResourceInformation = function () {
        // "Resources:" label
        var resInfoTitle = this.addNewLabel("".concat(DIAG_CITY_RESINFO, "-title"), "  Resources:", false, true);
        resInfoTitle.textHorizontalAlignment = GUI.TextBlock.HORIZONTAL_ALIGNMENT_LEFT;
        resInfoTitle.fontFamily = GUI_CONST.FONT_HIGHLIGHTED;
        // resource info containter
        var resInfoOutter = new GUI.Rectangle("".concat(DIAG_CITY_RESINFO, "-outter"));
        resInfoOutter.width = GUI_CONST.DEFAULT_INNER_WIDTH;
        resInfoOutter.height = GUI_CONST.DEFAULT_LABEL_HEIGHT;
        resInfoOutter.thickness = 0;
        var resInfoBg = new GUI.Image("".concat(DIAG_CITY_RESINFO, "-bg"), GUI_CONST.BG_PAPER);
        resInfoBg.width = GUI_CONST.DEFAULT_INNER_WIDTH;
        resInfoBg.height = GUI_CONST.DEFAULT_LABEL_HEIGHT;
        resInfoOutter.addControl(resInfoBg);
        // most resources
        var resInfoRow = new GUI.StackPanel(DIAG_CITY_RESINFO);
        resInfoRow.height = GUI_CONST.DEFAULT_LABEL_HEIGHT;
        resInfoRow.isVertical = false;
        resInfoRow.horizontalAlignment;
        resInfoRow.width = GUI_CONST.DEFAULT_INNER_WIDTH;
        // add row to panel
        resInfoOutter.addControl(resInfoRow);
        // well-stocked resources
        for (var _i = 0, _a = this.city.tradeNode.getMostStored(); _i < _a.length; _i++) {
            var storedRes = _a[_i];
            resInfoRow.addControl(this.makeResourceStorageElement(storedRes, true));
        }
        this.addControl(resInfoOutter, true);
        this.addNewDivider(true);
    };
    CityDialog.prototype.makeResourceStorageElement = function (storedRes, colorByAmount) {
        var elemId = "diag-res-max-".concat(storedRes.resId);
        // create outter ractangle
        var container = new GUI.Rectangle(elemId);
        container.height = GUI_CONST.DEFAULT_LABEL_HEIGHT;
        container.color = "black";
        container.thickness = 0;
        // container.background = "black";
        var widthInPx = "".concat(Math.floor(400 / 3), "px");
        container.width = widthInPx;
        // get resource label info
        var resObj = globalThis.RESOURCE_TYPES.get(storedRes.resId);
        var colorHexStr = BABYLON.Color3.FromArray(resObj.color).toHexString();
        if (colorByAmount) {
            /* when a resource should be colored by its amount in storage,
               paint scarce ones red and well-stocked ones green */
            var amountColorRgb = DawnbringerPalette.DIMGRAY;
            if (storedRes.amount > 15) {
                amountColorRgb = DawnbringerPalette.DELL;
            }
            else if (storedRes.amount <= 5) {
                amountColorRgb = DawnbringerPalette.BROWN;
            }
            colorHexStr = BABYLON.Color3.FromArray(amountColorRgb).toHexString();
        }
        var amount = Math.round(storedRes.amount);
        var labelText = "".concat(amount, " ").concat(resObj.name);
        // create resource label
        var label = new GUI.TextBlock("".concat(elemId, "-label"), labelText);
        container.addControl(label);
        label.width = widthInPx;
        label.height = GUI_CONST.DEFAULT_ELEMENT_HEIGHT;
        label.fontSize = GUI_CONST.TEXTSIZE_NORMAL;
        label.fontFamily = GUI_CONST.FONT_HIGHLIGHTED;
        label.color = colorHexStr;
        return container;
    };
    // on child click
    CityDialog.prototype.onButtonClick = function (btnId) {
        _super.prototype.onButtonClick.call(this, btnId);
        // resource list btn
        if (btnId == DIAG_CITY_BTN_TRADE) {
            this.switchToDialog(new ResourceListDialog(this.city.tradeNode));
        }
        // city recruit btn
        else if (btnId == DIAG_CITY_BTN_RECRUIT) {
            var playerArmy = globalThis.PLAYER.army;
            var recruitDiag = new RecruitmentDialog(playerArmy);
            globalThis.GUI.showDialog(recruitDiag);
        }
        // city command btn
        else if (btnId == DIAG_CITY_BTN_COMMAND) {
            this.log("COMMAND");
        }
    };
    CityDialog.prototype.onElementClicked = function (elemId) {
        var success = _super.prototype.onElementClicked.call(this, elemId);
        var city = this.city;
        if (!success) {
            return false;
        }
        // buildings
        if (elemId == 'datarow-Buildings') {
            this.switchToDialog(new CityBuildingsDialog(city));
        }
        // connections
        else if (elemId == 'datarow-Connections') {
            this.switchToDialog(new CityConnectionsDialog(city));
        }
        // militia
        else if (elemId == 'datarow-Militia') {
            this.switchToDialog(new ArmiesDialog(this.city.pos));
        }
        // pops
        else if (elemId == 'datarow-Population') {
            this.switchToDialog(new CityPopulationDialog(city));
        }
        // else
        else {
            this.log("element ".concat(elemId, " clicked but no action is connected"));
        }
        return true;
    };
    // EVENT LISTENING
    CityDialog.prototype.onEvent = function (ev) {
        _super.prototype.onEvent.call(this, ev);
        // ELEMENT CLICKED
        if (ev.id = EV.ID_GUI_ELEM_CLICKED) {
            var clickedElemId = ev.properties.getStr("elemId");
            if (clickedElemId) {
                this.onElementClicked(clickedElemId);
            }
        }
    };
    return CityDialog;
}(EmptyDialog));
export { CityDialog };
/* == CONSTANTS == */
// city dialog
export var DIAG_CITY = "diag-city";
export var DIAG_CITY_RESINFO = "diag-city-resinfo";
export var DIAG_CITY_BTN_TRADE = "diag-city-btn-trade";
export var DIAG_CITY_BTN_RECRUIT = "diag-city-btn-recruit";
export var DIAG_CITY_BTN_COMMAND = "diag-city-btn-command";
export var DIAG_CITY_LBL_LEADER = "diag-city-leader";
export var DIAG_CITY_LBL_CONNECTIONS = "diag-city-connections";
export var DIAG_CITY_LBL_MILITIA = "diag-city-militia";
export var DIAG_CITY_LBL_BUILDINGS = "diag-city-buildings";
export var DIAG_CITY_LBL_POPS = "diag-city-growth";
//# sourceMappingURL=city-dialog.js.map