var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (Object.prototype.hasOwnProperty.call(b, p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        if (typeof b !== "function" && b !== null)
            throw new TypeError("Class extends value " + String(b) + " is not a constructor or null");
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
// trade
import { StoredResource } from "../../entities/trade/resource-storage";
// gui
import { GuiTools } from "../common/gui-tools";
import * as GUI_CONST from '../common/constants';
import { EmptyDialog } from "./common/base-dialog";
import { GuiRectangle } from "../common/rectangle";
// babylon
import * as GUI from 'babylonjs-gui';
/* == CITY TRADE TRANSFER == */
var CityTransferDialog = /** @class */ (function (_super) {
    __extends(CityTransferDialog, _super);
    function CityTransferDialog(location, resId) {
        var _this = _super.call(this, DIAG_TRANSFER, true, "Transfer") || this;
        _this.resId = resId;
        // set location & trade node property
        _this.location = location;
        _this.tradeNode = location.tradeNode;
        return _this;
    }
    CityTransferDialog.prototype.build = function () {
        _super.prototype.build.call(this);
    };
    Object.defineProperty(CityTransferDialog.prototype, "player", {
        get: function () {
            return globalThis.PLAYER;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(CityTransferDialog.prototype, "playerArmy", {
        get: function () {
            return this.player.army;
        },
        enumerable: false,
        configurable: true
    });
    CityTransferDialog.prototype.addCustomContent = function () {
        var resource = CityTransferDialog.GetResById(this.resId);
        // res image
        this.addResourceImage();
        // res name
        this.addNewDataRow("name", resource.name, "name");
        // storage
        var inCity = this.location.tradeNode.amountOf(this.resId);
        var cityStorageText = inCity.toFixed(1);
        this.cityStorageRow = this.addNewDataRow("City", cityStorageText, "storage");
        this.playerStorageRow = this.addNewDataRow("Player", "" + this.player.amountInStorage(this.resId), "trader");
        // buttons
        this.addBuySellButtons();
    };
    CityTransferDialog.prototype.addResourceImage = function () {
        // make container
        var resImgContainerId = DIAG_TRANSFER_RESIMG_ID + "-container";
        this.resImgContainer = new GuiRectangle(resImgContainerId);
        this.resImgContainer.setSize(GUI_CONST.DEFAULT_INNER_WIDTH, DIAG_TRANSFER_RESIMG_SIZE);
        // make container bg
        var resImgBgId = DIAG_TRANSFER_RESIMG_ID + "-bg";
        this.resImgBackground = new GUI.Image(resImgBgId, GUI_CONST.BG_PAPER);
        GuiTools.setSizeFrom(this.resImgBackground, this.resImgContainer);
        // make image
        var resImgUrl = CityTransferDialog.GetResImgPath(this.resId);
        this.resImg = new GUI.Image(DIAG_TRANSFER_RESIMG_ID, resImgUrl);
        this.resImg.width = DIAG_TRANSFER_RESIMG_SIZE;
        this.resImg.height = DIAG_TRANSFER_RESIMG_SIZE;
        // add to dialog
        this.resImgContainer.addControl(this.resImgBackground);
        this.resImgContainer.addControl(this.resImg);
        this.addControl(this.resImgContainer, true);
    };
    CityTransferDialog.prototype.addBuySellButtons = function () {
        // sett button
        var storedByPlayer = this.player.getStoredRes(this.resId);
        var isInPlayerStorage = storedByPlayer && storedByPlayer.amount >= 1;
        var cityPrice = this.tradeNode.priceOf(this.resId);
        if (isInPlayerStorage) {
            var boughtPrice = storedByPlayer.price;
            var playerSellProfit = cityPrice - boughtPrice;
            this.addNewButton(DIAG_TRANSFER_BTN_SELL, "-1 (Prf.: ".concat(playerSellProfit, "o)"));
        }
        // buy button
        var isInCityStorage = this.tradeNode.hasEnough(this.resId, 1);
        if (isInCityStorage) {
            this.addNewButton(DIAG_TRANSFER_BTN_BUY, "+1 (Price: ".concat(cityPrice.toFixed(1), "o)"));
        }
    };
    CityTransferDialog.prototype.onButtonClick = function (btnId) {
        _super.prototype.onButtonClick.call(this, btnId);
        if (btnId == DIAG_TRANSFER_BTN_BUY) {
            this.buyOnce();
        }
        else if (btnId == DIAG_TRANSFER_BTN_SELL) {
            this.sellOnce();
        }
        this.updateCustomContent();
    };
    CityTransferDialog.prototype.updateCustomContent = function () {
        // city storage
        var cityStorageAmount = this.location.tradeNode.amountOf(this.resId).toFixed(1);
        this.cityStorageRow.valueLabel.text = "" + cityStorageAmount;
        // player storage
        var playerStorageAmount = globalThis.PLAYER.storage.amountOf(this.resId).toFixed(1);
        this.playerStorageRow.valueLabel.text = "" + playerStorageAmount;
    };
    CityTransferDialog.GetResById = function (resId) {
        var resource = globalThis.RESOURCE_TYPES.get(resId);
        if (!resource) {
            console.error("<Dialog> no resObj for ".concat(resId));
            return;
        }
        if (globalThis.TRADE && globalThis.TRADE.hasSpecialResLoaded) {
            resource = globalThis.TRADE.getRes(resId);
        }
        return resource;
    };
    CityTransferDialog.GetResImgPath = function (resId) {
        return "res/img/icons/res/big/shadowed_".concat(resId, ".png");
    };
    CityTransferDialog.prototype.buyOnce = function () {
        var storedRes = this.location.tradeNode.getStoredResource(this.resId);
        var success = this.location.tradeNode.remove(this.resId, 1);
        if (storedRes && success) {
            this._buyResource(this.resId, storedRes.price);
        }
    };
    CityTransferDialog.prototype._buyResource = function (resId, price) {
        // exchange coins
        globalThis.PLAYER.coins -= price;
        // add to storage
        globalThis.PLAYER.storage.add(this.resId, 1, price);
        var storedRes = new StoredResource(resId, 1, price);
        globalThis.PLAYER.army.storedResources.push(storedRes);
        // log & inform trade manager
        this.log("bought 1 ".concat(resId, " from ").concat(this.location.name, " for ").concat(price, "c"));
        globalThis.TRADE.onPlayerCoinsChange();
    };
    CityTransferDialog.prototype.sellOnce = function () {
        var enoughInStorage = globalThis.PLAYER.storage.hasEnough(this.resId, 1);
        if (!enoughInStorage) {
            this.warn("cant sell more \"".concat(this.resId, "\": not enough in player storage"));
            return;
        }
        var storedRes = this.player.getStoredRes(this.resId);
        var canRemove = false;
        if (storedRes) {
            canRemove = globalThis.PLAYER.storage.remove(this.resId, 1);
        }
        if (canRemove) {
            this._sellResource(this.resId, storedRes.price);
        }
    };
    CityTransferDialog.prototype._sellResource = function (resId, playerPrice) {
        // remove from player storage & add to city market
        globalThis.PLAYER.army.removeSingleStoredResource(resId);
        this.location.tradeNode.add(this.resId, 1);
        // exchange coins
        var localPrice = this.location.tradeNode.priceOf(resId);
        globalThis.PLAYER.coins += localPrice;
        this.location.money -= localPrice;
        // log & inform trade manager
        this.log("sold 1 ".concat(resId, " bought for ").concat(playerPrice, "c for").concat(localPrice, "c"));
        globalThis.TRADE.onPlayerCoinsChange();
    };
    return CityTransferDialog;
}(EmptyDialog));
export { CityTransferDialog };
/* == CONSTANTS == */
// transfer dialog
export var DIAG_TRANSFER = "diag-transfer";
export var DIAG_TRANSFER_BTN_BUY = "btn-transfer-buy";
export var DIAG_TRANSFER_BTN_SELL = "btn-transfer-sell";
export var DIAG_TRANSFER_RESIMG_ID = DIAG_TRANSFER + "-img-res";
export var DIAG_TRANSFER_RESIMG_SIZE = "200px";
//# sourceMappingURL=city-transfer-dialog.js.map