var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (Object.prototype.hasOwnProperty.call(b, p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        if (typeof b !== "function" && b !== null)
            throw new TypeError("Class extends value " + String(b) + " is not a constructor or null");
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
// gui
import * as GUI_CONST from '../common/constants';
// babylon
import * as GUI from 'babylonjs-gui';
var Switcher = /** @class */ (function (_super) {
    __extends(Switcher, _super);
    function Switcher(name, options) {
        var _this = _super.call(this, name) || this;
        // properties
        _this.currentOptionNum = 0;
        // properties
        _this.isVertical = false;
        _this.options = options;
        // initialize children
        _this.createInnerControls();
        return _this;
    }
    /* creates in this order: left btn, label containeer, label bg, label textblock, right btn */
    Switcher.prototype.createInnerControls = function () {
        this.addSwitcherButton(false);
        this.addSwitcherLabel();
        this.addSwitcherButton(true);
    };
    /* SWITCHER MIDDLE CONTENT */
    Switcher.prototype.addSwitcherLabelContainer = function () {
        // container rectangle
        this.labelContainer = new GUI.Rectangle(SWITCHER_LABEL_CONTAINER_ID);
        this.labelContainer.widthInPixels = SWITCHER_LABEL_WIDTH;
        this.labelContainer.thickness = 0;
        // background image
        this.labelBackground = new GUI.Image(SWITCHER_LABEL_BG_ID, GUI_CONST.BG_PAPER);
        this.labelBackground.widthInPixels = SWITCHER_LABEL_WIDTH;
        this.labelContainer.addControl(this.labelBackground);
    };
    Switcher.prototype.addSwitcherLabel = function () {
        // create container & background
        this.addSwitcherLabelContainer();
        // create inner texblock label
        var startingOptionStr = this.options[0];
        this.label = new GUI.TextBlock(SWITCHER_LABEL_ID, startingOptionStr);
        this.label.widthInPixels = SWITCHER_LABEL_WIDTH;
        this.labelContainer.addControl(this.label);
        this.addControl(this.labelContainer);
    };
    /* SWITCHER ARROW BUTTONS */
    Switcher.prototype.addSwitcherButton = function (isRight) {
        if (!isRight) {
            isRight = false;
        }
        // create button
        var btnId = Switcher.GetSwitcherBtnId(isRight);
        var btnText = Switcher.GetSwitcherButtonText(isRight);
        var switcherBtn = GUI.Button.CreateSimpleButton(btnId, btnText);
        // set bg and size
        switcherBtn.thickness = 0;
        switcherBtn.background = "brown";
        switcherBtn.widthInPixels = SWITCHER_BTN_WIDTH;
        this.addArrowOnClick(this, switcherBtn, !isRight);
        this.addControl(switcherBtn);
        // save in switcher variable
        if (isRight) {
            this.btnRight = switcherBtn;
        }
        else {
            this.btnLeft = switcherBtn;
        }
    };
    /* returns either "<" or ">" depending on whether button is to the left or right */
    Switcher.GetSwitcherButtonText = function (isRight) {
        var btnDirNum = 0;
        if (isRight == true) {
            btnDirNum = 1;
        }
        return SWITCHER_BTN_TEXTS[btnDirNum];
    };
    /* returns either "l" or "r" depending on whether button is to the left or right */
    Switcher.GetSwitcherDirChar = function (isRight) {
        if (isRight == true) {
            return SWITCHER_DIR_STR[0];
        }
        return SWITCHER_DIR_STR[1];
    };
    Switcher.GetSwitcherBtnId = function (isRight) {
        return SWITCHER_BTN_PREFIX + Switcher.GetSwitcherDirChar(isRight);
    };
    Switcher.prototype.addArrowOnClick = function (parent, elem, isLeft) {
        elem.onPointerDownObservable.add(function () {
            parent.onArrowClick(isLeft);
        });
    };
    Switcher.prototype.onArrowClick = function (isLeft) {
        console.log("<Switcher ".concat(this.uniqueId, "> ARROW left?").concat(isLeft));
        if (isLeft) {
            this.switchBackwards();
        }
        else {
            this.switchForwards();
        }
    };
    Switcher.prototype.switchBackwards = function () {
        this.currentOptionNum -= 1;
        if (this.currentOptionNum < 0) {
            this.currentOptionNum = this.options.length - 1;
        }
        this.onChange();
    };
    Switcher.prototype.switchForwards = function () {
        this.currentOptionNum += 1;
        if (this.currentOptionNum >= this.options.length) {
            this.currentOptionNum = 0;
        }
        this.onChange();
    };
    Switcher.prototype.onChange = function () {
        // update label text
        var newText = this.options[this.currentOptionNum];
        this.label.text = newText;
        // log
        var _tag = "<Switcher \"".concat(this.name, "\"> ");
        console.log(_tag + 'changed to option: ' + newText);
        // inform dialog
        var currentDialog = globalThis.DIALOG;
        if (currentDialog) {
            console.log(_tag + 'send to dialog');
            currentDialog.onSwitcherChange(this.name, newText);
        }
        else {
            console.warn(_tag + 'cant sent to dialog');
        }
    };
    return Switcher;
}(GUI.StackPanel));
export { Switcher };
/* == CONSTANTS == */
// buttons
export var SWITCHER_DIR_STR = ["l", "r"];
export var SWITCHER_BTN_PREFIX = "switcher-btn-";
export var SWITCHER_BTN_TEXTS = ["<", ">"];
export var SWITCHER_BTN_WIDTH = 40;
export var SWITCHER_BTN_OUTTER_WIDTH = 60;
// label
export var SWITCHER_LABEL_ID = "switcher-label";
export var SWITCHER_LABEL_WIDTH = 400 - 2 * (SWITCHER_BTN_WIDTH);
export var SWITCHER_LABEL_CONTAINER_ID = "switcher-label-container";
export var SWITCHER_LABEL_BG_ID = "switcher-label-bg";
//# sourceMappingURL=switcher.js.map