var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (Object.prototype.hasOwnProperty.call(b, p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        if (typeof b !== "function" && b !== null)
            throw new TypeError("Class extends value " + String(b) + " is not a constructor or null");
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
import { ColorPalette } from "../../../helpers/colorpalette";
import * as EV from "../../game-events";
// gui
import * as GUI_CONST from '../common/constants';
import { SideBarMenu } from "./common/side-bar-menu";
import { InlineIcon } from "../elements/icons";
import { GuiTools } from "../common/gui-tools";
// gui: dialogs
import { FamilyCaravansDialog } from "../dialogs/family-caravans-dialog";
import { GameMenuDialog } from "../dialogs/game-menu-dialog";
// babylon
import * as GUI from 'babylonjs-gui';
var TopBarMenu = /** @class */ (function (_super) {
    __extends(TopBarMenu, _super);
    function TopBarMenu() {
        var _this = _super.call(this, MENU_ID, false, false) || this;
        _this.elements = {};
        _this.innerLabel = null;
        _this.menuButtons = [];
        // create children
        _this.createMenuButtons();
        _this.addMenuItems();
        return _this;
    }
    /* INITIALIZE SELF & BACKGROUND IMAGE */
    TopBarMenu.prototype.createMenuButtons = function () {
        this.createTopBarButton(false);
        this.createTopBarButton(true);
    };
    /* CREATE MENU ELEMENTS */
    TopBarMenu.prototype.addMenuItems = function () {
        this.addIconValueLabel("family", "Lowborn", "name_bright"); // player family name
        this.addIconValueLabel("coins", "500", "money_bright", true); // player coins
        this.addIconValueLabel("cargo", "0", "storage", true); // player free cargo
        this.addIconValueLabel("pos", "0", "tile_bright", true); // player pos
        this.addIconValueLabel("turn", "0", "turn_bright", true); // turn number
        this.width = "100%";
    };
    /* CREATE MENU ELEMENT LABELS & ICONS */
    TopBarMenu.prototype.setLabelFontStyle = function (label) {
        label.fontFamily = GUI_CONST.FONT_HIGHLIGHTED;
        label.fontSize = GUI_CONST.TEXTSIZE_NORMAL;
        label.color = ColorPalette.NameAsHexStr(GUI_CONST.MENU_TEXT_COLOR_NAME);
        return label;
    };
    TopBarMenu.prototype.addKeyValueLabel = function (key, text, value) {
        this._createMenuItemContainer(key);
        var keyLabel = this._createKeyLabel(key, text);
        var valueLabel = this._createValueLabel(key, value);
        this.elements[key] = this.container;
        return this.container;
    };
    TopBarMenu.prototype.addIconValueLabel = function (key, valueText, customIcon, shortValue) {
        this._createMenuItemContainer(key);
        var keyIcon = this._createKeyIcon(key, customIcon);
        var valueLabel = this._createValueLabel(key, valueText, shortValue);
        this.elements[key] = this.container;
        return this.container;
    };
    TopBarMenu.prototype._createMenuItemContainer = function (key) {
        var elementId = ID_ELEMENT_PREFIX + key;
        this.container = new GUI.StackPanel(elementId);
        this.container.width = ELEMENT_FULL_WIDTH;
        this.container.height = GUI_CONST.MENU_HEIGHT_SMALL;
        this.container.isVertical = false;
        this.outterLayout.addControl(this.container);
    };
    TopBarMenu.prototype._createKeyLabel = function (key, keyText) {
        var keyLabel = this.makeElementLabel(key, keyText, true);
        this.setLabelFontStyle(keyLabel);
        this.container.addControl(keyLabel);
        return keyLabel;
    };
    TopBarMenu.prototype._createKeyIcon = function (key, customIcon) {
        var iconId = TopBarMenu.GetKeyIconId(key);
        var iconImageName = key;
        if (customIcon) {
            iconImageName = customIcon;
        }
        var icon = new InlineIcon(iconId, iconImageName);
        // add to inner layout
        this.container.addControl(icon);
        return icon;
    };
    TopBarMenu.prototype._createValueLabel = function (key, valueText, isShort) {
        var valueLabel = this.makeElementLabel(key, valueText, false);
        this.setLabelFontStyle(valueLabel);
        if (isShort) {
            this.container.width = ELEMENT_VALUE_WIDTH_SHORT;
            this.width = ELEMENT_SHORT_WIDTH;
        }
        this.container.addControl(valueLabel);
        return valueLabel;
    };
    TopBarMenu.prototype.setElementValueText = function (key, value) {
        var valueLabel = this.getElementLabel(key, false);
        if (valueLabel) {
            valueLabel.text = value;
        }
    };
    /* returns babylon TextBlock component of key or value label
        key: element key as string (p.e. "coins") */
    TopBarMenu.prototype.getElementLabel = function (key, isKey) {
        // look for value label?
        var isValue = true;
        if (isKey && isKey == true) {
            isValue = false;
        }
        var labelId = TopBarMenu.GetLabelId(key, isValue);
        for (var _i = 0, _a = this.getDescendants(); _i < _a.length; _i++) {
            var topBarChild = _a[_i];
            if (topBarChild.name == labelId) {
                return topBarChild;
            }
        }
        return null;
    };
    TopBarMenu.prototype.createTopBarButton = function (isGameMenu) {
        var buttonId = ID_BTN_TRADERS;
        var btnHAlign = GUI_CONST.ALIGN_H_LEFT;
        if (isGameMenu && isGameMenu == true) {
            buttonId = ID_BTN_MENU;
            btnHAlign = GUI_CONST.ALIGN_H_RIGHT;
        }
        var fullId = ID_BTN_PREFIX + buttonId;
        var btnText = buttonId.toLocaleUpperCase();
        // create label & set alignment
        var btnLabelElem = GuiTools.addNewGuiLabel(this, fullId, btnText, true, false, true);
        this.menuButtons.push(btnLabelElem);
        btnLabelElem.horizontalAlignment = btnHAlign;
        // style font
        btnLabelElem.color = ColorPalette.NameAsHexStr("LIGHTSTEEL");
        // btnLabelElem.fontFamily = GUI_CONST.FONT_PARAGRAPH;
        // btnLabelElem.fontWeight = "bold";
    };
    TopBarMenu.prototype.makeElementLabel = function (key, text, isKey) {
        var labelText = text + ": ";
        var labelTextAlign = GUI_CONST.ALIGN_TEXT_H_RIGHT;
        // is a value lavel?
        if (!isKey) {
            labelText = text;
            labelTextAlign = GUI_CONST.ALIGN_TEXT_H_LEFT;
        }
        var isValue = !isKey;
        var labelId = TopBarMenu.GetLabelId(key, isValue);
        var label = new GUI.TextBlock(labelId, labelText);
        // sizing
        label.width = this.getLabelWidth(key, isValue);
        label.height = GUI_CONST.MENU_HEIGHT_SMALL;
        label.textHorizontalAlignment = labelTextAlign;
        return label;
    };
    /* MENU UPDATE */
    TopBarMenu.prototype.updateAll = function () {
        if (this.getElementLabel("coins")) {
            this.setElementValueText("coins", "".concat(this.playerCoins));
        }
        if (this.getElementLabel("cargo")) {
            this.setElementValueText("cargo", "".concat(this.freePlayerStorage));
        }
        if (this.getElementLabel("family")) {
            this.setElementValueText("family", this.playerFamilyName);
        }
        if (this.getElementLabel("turn")) {
            this.setElementValueText("turn", "".concat(this.turnNum));
        }
        if (this.getElementLabel("pos")) {
            this.setElementValueText("pos", this.playerPosStr);
        }
    };
    Object.defineProperty(TopBarMenu.prototype, "playerCoins", {
        /* VALUE GETTERS */
        get: function () {
            if (globalThis.PLAYER) {
                return globalThis.PLAYER.coins;
            }
            return 0;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(TopBarMenu.prototype, "freePlayerStorage", {
        get: function () {
            if (globalThis.PLAYER && globalThis.PLAYER.army) {
                return globalThis.PLAYER.army.freeStorageSpace;
            }
            return 0;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(TopBarMenu.prototype, "playerFamilyName", {
        get: function () {
            if (globalThis.PLAYER && globalThis.PLAYER.family) {
                return globalThis.PLAYER.family.name;
            }
            return "Lowborn";
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(TopBarMenu.prototype, "turnNum", {
        get: function () {
            if (globalThis.TURNS) {
                return globalThis.TURNS.active.num;
            }
            return 0;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(TopBarMenu.prototype, "playerPosStr", {
        get: function () {
            if (globalThis.PLAYER && globalThis.PLAYER.pos) {
                return "".concat(globalThis.PLAYER.pos.asString());
            }
            return "";
        },
        enumerable: false,
        configurable: true
    });
    TopBarMenu.prototype.getLabelWidth = function (key, isValueLabel) {
        if (!isValueLabel) {
            return ELEMENT_KEY_WIDTH;
        }
        return ELEMENT_VALUE_WIDTH;
    };
    /* STATIC ELEMENT ID GETTERS */
    TopBarMenu.GetValueLabelId = function (key) {
        return ID_ELEMENT_PREFIX + key + "-valuelabel";
    };
    TopBarMenu.GetKeyLabelId = function (key) {
        return ID_ELEMENT_PREFIX + key + "-keylabel";
    };
    TopBarMenu.GetLabelId = function (key, isValueLabel) {
        if (!isValueLabel) {
            return TopBarMenu.GetKeyLabelId(key);
        }
        return TopBarMenu.GetValueLabelId(key);
    };
    TopBarMenu.GetKeyIconId = function (key) {
        return ID_ELEMENT_PREFIX + key + "-keyicon";
    };
    TopBarMenu.GetBtnIdFromElementId = function (elemId) {
        return elemId.replace(ID_BTN_PREFIX, "");
    };
    /* EVENT HANDLING */
    TopBarMenu.prototype.onEvent = function (ev) {
        _super.prototype.onEvent.call(this, ev);
        if (ev.id == EV.ID_PLAYER_MOVED) {
            this.updateAll();
        }
        if (ev.id == EV.ID_GUI_ELEM_CLICKED) {
            var elemId = ev.properties.getStr("elemId");
            this.onElementClicked(elemId);
        }
        return true;
    };
    TopBarMenu.prototype.onElementClicked = function (elemId) {
        if (!elemId.startsWith("menu")) {
            return false;
        }
        this.log("click on menues element \"".concat(elemId, "\""));
        // click on top bar button? (Traders or Game Menu)?
        if (elemId.startsWith(ID_BTN_PREFIX)) {
            var btnId = TopBarMenu.GetBtnIdFromElementId(elemId);
            this.log("CLICK on button \"".concat(btnId, "\""));
            // click on Traders button? show FamilyCaravansDialog for player
            if (btnId == ID_BTN_TRADERS) {
                var caravansDiag = new FamilyCaravansDialog();
                globalThis.GUI.showDialog(caravansDiag);
                return true;
            }
            else if (btnId == ID_BTN_MENU) {
                var gameMenuDiag = new GameMenuDialog();
                globalThis.GUI.showDialog(gameMenuDiag);
                return true;
            }
        }
        return false;
    };
    return TopBarMenu;
}(SideBarMenu));
export { TopBarMenu };
/* == TOP BAR CONSTANTS == */
// gui ids
export var MENU_ID = "top";
export var ID_MENU_PREFIX = "menu-" + MENU_ID + "-";
export var ID_MENU_OUTTER = ID_MENU_PREFIX + "outter";
export var ID_MENU_BG = ID_MENU_PREFIX + "bg";
export var ID_MENU_LAYOUT = ID_MENU_PREFIX + "layout";
export var ID_ELEMENT_PREFIX = ID_MENU_PREFIX + "element-";
export var ID_INNER_LABEL = ID_MENU_LAYOUT + "-label";
export var ID_INNER_ICON = ID_MENU_LAYOUT + "-icon";
// top bar traders & menu buttons
export var ID_BTN_PREFIX = ID_MENU_PREFIX + "btn-";
export var ID_BTN_TRADERS = "traders";
export var ID_BTN_MENU = "menu";
export var MENU_BUTTON_IDS = [ID_BTN_TRADERS, ID_BTN_MENU];
// element children size
export var ELEMENT_SHORT_X_MINUS = 30;
export var ELEMENT_ICON_PADDING = "5px";
export var ELEMENT_KEY_WIDTH_PX = 90;
export var ELEMENT_KEY_WIDTH = "".concat(ELEMENT_KEY_WIDTH_PX, "px");
export var ELEMENT_VALUE_WIDTH_PX = 130;
export var ELEMENT_VALUE_WIDTH_SHORT_PX = ELEMENT_VALUE_WIDTH_PX - ELEMENT_SHORT_X_MINUS;
export var ELEMENT_VALUE_WIDTH_SHORT = "".concat(ELEMENT_VALUE_WIDTH_SHORT_PX, "px");
export var ELEMENT_VALUE_WIDTH = "".concat(ELEMENT_VALUE_WIDTH_PX, "px");
// menu size 
export var ELEMENT_FULL_WIDTH_PX = ELEMENT_KEY_WIDTH_PX + ELEMENT_VALUE_WIDTH_PX;
export var ELEMENT_FULL_WIDTH = "".concat(ELEMENT_FULL_WIDTH_PX, "px");
export var ELEMENT_SHORT_WIDTH_PX = ELEMENT_FULL_WIDTH_PX - ELEMENT_SHORT_X_MINUS;
export var ELEMENT_SHORT_WIDTH = "".concat(ELEMENT_SHORT_WIDTH_PX, "px");
//# sourceMappingURL=top-bar.js.map