var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (Object.prototype.hasOwnProperty.call(b, p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        if (typeof b !== "function" && b !== null)
            throw new TypeError("Class extends value " + String(b) + " is not a constructor or null");
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
import { MarketTrade } from '../entities/trade/market-trade';
import { Road } from '../world/road';
// manager
import { AbstractManager } from "./abstract-manager";
/* == TRADE MANAGER == */
var TradeManager = /** @class */ (function (_super) {
    __extends(TradeManager, _super);
    function TradeManager() {
        var _this = _super.call(this, "Trade") || this;
        _this.worldTradeHistory = [];
        // roads
        _this.roads = [];
        _this.lastRoadNum = -1;
        return _this;
    }
    TradeManager.prototype.onCitiesLoaded = function () {
        this.createRoadNetwork();
    };
    TradeManager.prototype.overrideSpecialResources = function () {
    };
    /* called when city delivery event is fired */
    TradeManager.prototype.onDelivery = function (source, target, storedRes) {
        var turnNum = globalThis.TURNS.turnNum();
        var marketTrade = new MarketTrade(turnNum, storedRes.resId, storedRes.profit, target, source);
        // add to world trade history
        this.worldTradeHistory.push(marketTrade);
        // increase city connection counter
        var connection = globalThis.CONN.fromTo(source, target);
        connection.addToTradeHistory(marketTrade, true);
        // add to trade history
        source.tradeHistory.add(marketTrade, true);
        target.tradeHistory.add(marketTrade, false);
    };
    TradeManager.prototype.onPlayerTrade = function (isSell, city, storedRes, profit) {
    };
    Object.defineProperty(TradeManager.prototype, "hasSpecialResLoaded", {
        get: function () {
            return globalThis.SPECIAL_RES && globalThis.SPECIAL_RES.length > 0;
        },
        enumerable: false,
        configurable: true
    });
    /* inform gui over changed player monex */
    TradeManager.prototype.onPlayerCoinsChange = function () {
        globalThis.GUI.updateTopBar();
    };
    TradeManager.prototype.getRes = function (resId) {
        if (resId.startsWith("X")) {
            var xResNum = parseInt(resId.replace("X", ""));
            return globalThis.SPECIAL_RES[xResNum];
        }
        else {
            return globalThis.RESOURCE_TYPES.get(resId);
        }
    };
    TradeManager.prototype.getResources = function () {
        var allResTypes = [];
        for (var _i = 0, _a = RESOURCE_TYPES.getIds(); _i < _a.length; _i++) {
            var resId = _a[_i];
            allResTypes.push(this.getRes(resId));
        }
        return allResTypes;
    };
    TradeManager.prototype.getResByName = function (resName) {
        for (var _i = 0, _a = globalThis.RESOURCE_TYPES.getAll(); _i < _a.length; _i++) {
            var tradeRes = _a[_i];
            if (tradeRes.name == resName) {
                return tradeRes;
            }
        }
        return;
    };
    /* returns the resource name string ("Grain") of a resource with given resId */
    TradeManager.prototype.getResName = function (resId) {
        var resObj = this.getRes(resId);
        return resObj.name;
    };
    TradeManager.prototype.getMarkets = function () {
        var markets = [];
        // city markets
        var cities = globalThis.CITIES.getAll();
        if (cities.length >= 0) {
            this.error("cant get markets; cities not loaded yet");
        }
        // add market to list
        for (var _i = 0, cities_1 = cities; _i < cities_1.length; _i++) {
            var city = cities_1[_i];
            markets.push(city.tradeNode);
        }
        return markets;
    };
    /* CITY CONNECTIONS */
    TradeManager.prototype.getCityConnection = function (from, to) {
        var connection = from.getConnectionTo(to);
        if (!connection) {
            this.warn("cant get city connection: ".concat(from.name, " to ").concat(to.name));
            return;
        }
        return connection;
    };
    /* ROADS */
    TradeManager.prototype.createRoadNetwork = function () {
        var roadCounter = 0;
        // create road objects
        for (var _i = 0, _a = globalThis.CITIES.cities; _i < _a.length; _i++) {
            var sourceCity = _a[_i];
            var sourceLocation = sourceCity;
            for (var _b = 0, _c = sourceLocation.connections; _b < _c.length; _b++) {
                var connection = _c[_b];
                var targetLocation = connection.target;
                this.addRoad(sourceLocation, targetLocation);
                roadCounter += 1;
            }
        }
        this.log("".concat(roadCounter, " roads created"));
        // find road paths
        for (var _d = 0, _e = this.roads; _d < _e.length; _d++) {
            var road = _e[_d];
            road.findRoadPath();
        }
        this.log("created road paths");
        return roadCounter;
    };
    TradeManager.prototype.nextRoadNum = function () {
        var nextRoadNum = this.lastRoadNum + 1;
        this.lastRoadNum = nextRoadNum;
        return nextRoadNum;
    };
    TradeManager.prototype.getRoad = function (roadNum) {
        return this.roads[roadNum];
    };
    TradeManager.prototype.getRoadFromTo = function (fromCity, toCity) {
        var roadsFromTarget = fromCity.roads;
        for (var _i = 0, roadsFromTarget_1 = roadsFromTarget; _i < roadsFromTarget_1.length; _i++) {
            var startCityRoad = roadsFromTarget_1[_i];
            var isToTargetCity = startCityRoad.target.equals(toCity);
            if (isToTargetCity) {
                return startCityRoad;
            }
        }
        this.warn("cant find road between ".concat(fromCity.name, " & ").concat(toCity.name));
        return null;
    };
    TradeManager.prototype.getReturnRoad = function (roadObj) {
        return this.getRoadFromTo(roadObj.target, roadObj.start);
    };
    TradeManager.prototype.addRoad = function (startCity, targetCity) {
        var newRoad = new Road(startCity, targetCity, this.nextRoadNum());
        startCity.roads.push(newRoad);
        targetCity.roads.push(newRoad);
        this.roads.push(newRoad);
        return newRoad;
    };
    /* GETTER SHORTFORMS */
    TradeManager.prototype.GetRes = function (resId) { return this.getRes(resId); };
    TradeManager.prototype.GetResName = function (resId) { return this.getRes(resId).name; };
    return TradeManager;
}(AbstractManager));
export { TradeManager };
//# sourceMappingURL=trade-manager.js.map