var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (Object.prototype.hasOwnProperty.call(b, p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        if (typeof b !== "function" && b !== null)
            throw new TypeError("Class extends value " + String(b) + " is not a constructor or null");
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
import * as EV from "../game-events";
// gear
import { UnitWeapon, UnitArmour } from "../entities/unit/gear/gear";
import { UnitMount } from "../entities/unit/gear/mount";
// manager
import { AbstractManager } from "./abstract-manager";
var UnitGearManager = /** @class */ (function (_super) {
    __extends(UnitGearManager, _super);
    function UnitGearManager() {
        var _this = _super.call(this, "UnitGear") || this;
        // mounts
        _this._mountsJson = null;
        _this._mountIds = [];
        _this.mountTypes = [];
        // weapons
        _this._weaponsJson = null;
        _this._weaponIds = [];
        _this.weaponTypes = [];
        // armor
        _this._armorJson = null;
        _this._armorIds = [];
        _this.armorTypes = [];
        return _this;
    }
    /* MOUNT & GEAR BY ID */
    UnitGearManager.prototype.getWeaponById = function (weaponId) {
        var weaponNum = this._weaponIds.indexOf(weaponId);
        return this.weaponTypes[weaponNum];
    };
    UnitGearManager.prototype.getArmorById = function (armorId) {
        var armorNum = this._armorIds.indexOf(armorId);
        return this.armorTypes[armorNum];
    };
    UnitGearManager.prototype.getMountById = function (mountId) {
        var mountNum = this._mountIds.indexOf(mountId);
        return this.mountTypes[mountNum];
    };
    /* MOUNT & GEAR CREATION */
    UnitGearManager.prototype.addMountType = function (mount, overwriteId) {
        if (overwriteId) {
            mount.id = overwriteId;
        }
        this._mountIds.push(mount.id);
        this.mountTypes.push(mount);
        return mount;
    };
    UnitGearManager.prototype.addWeaponType = function (weapon, overwriteId) {
        if (overwriteId) {
            weapon.id = overwriteId;
        }
        this._weaponIds.push(weapon.id);
        this.weaponTypes.push(weapon);
        return weapon;
    };
    UnitGearManager.prototype.addArmorType = function (armor, overwriteId) {
        if (overwriteId) {
            armor.id = overwriteId;
        }
        this._armorIds.push(armor.id);
        this.armorTypes.push(armor);
        return armor;
    };
    /* GAME EVENT HANDLING */
    UnitGearManager.prototype.onEvent = function (ev) {
        _super.prototype.onEvent.call(this, ev);
        if (ev.id == EV.ID_FILE_JSON_LOADED) {
            if (ev.properties.getStr("shortPath") == "default-unit-mounts") {
                var languageJson = globalThis.FILES.jsonContent["default-unit-mounts"];
                this.onMountListLoaded(languageJson);
            }
        }
    };
    /* MOUNT & GEAR LOADING */
    UnitGearManager.prototype.onMountListLoaded = function (data) {
        this.log("mount list ".concat(data, " loaded!!"));
        this._mountsJson = data;
        for (var _i = 0, _a = Object.keys(data); _i < _a.length; _i++) {
            var mountId = _a[_i];
            var jsonValues = data[mountId];
            var baseHealth = parseInt(jsonValues["health"]);
            var baseMove = parseInt(jsonValues["mp"]);
            var baseCargo = parseInt(jsonValues["cargo"]);
            var isLand = jsonValues["isLand"];
            var terrModsStr = jsonValues["terrainModifiers"];
            var mount = new UnitMount(isLand, baseHealth, baseMove, terrModsStr, baseCargo);
            this.addMountType(mount, mountId);
        }
        this.checkMountsAndGearLoaded();
    };
    UnitGearManager.prototype.onGearListLoaded = function (gearListObj) {
        // weapons
        this._weaponsJson = gearListObj["weapons"];
        for (var _i = 0, _a = Object.keys(this._weaponsJson); _i < _a.length; _i++) {
            var weaponId = _a[_i];
            var jsonValues = this._weaponsJson[weaponId];
            var weaponObj = UnitWeapon.FromJson(jsonValues);
            this.addWeaponType(weaponObj, weaponId);
        }
        // armor
        this._armorJson = gearListObj["armor"];
        for (var _b = 0, _c = Object.keys(this._armorJson); _b < _c.length; _b++) {
            var armorId = _c[_b];
            var jsonValues = this._armorJson[armorId];
            var armorObj = UnitArmour.FromJson(jsonValues);
            this.addArmorType(armorObj, armorId);
        }
        // finish gear loading
        this.log("gear data loaded; initializing ".concat(this.weaponsCount, " weapons & ").concat(this.armorCount, " armor types"));
        this.checkMountsAndGearLoaded();
    };
    UnitGearManager.prototype.checkMountsAndGearLoaded = function () {
        if (this.mountTypes.length > 2 && this.weaponTypes.length > 2 && this.armorTypes.length > 2) {
            this.log("all mount, weapon & armour types loaded!");
            // inform faction manager
            globalThis.UNITS.onGearTypesLoaded();
            globalThis.FACTIONS.assignEmpireCapitals();
            // fire event
            var loadedEvent = new EV.UnitMountsAndGearLoadedEvent(this.mountAndGearCount);
            globalThis.EVENTS.fireEvent(loadedEvent, true);
        }
    };
    Object.defineProperty(UnitGearManager.prototype, "mountAndGearCount", {
        get: function () {
            return this.armorCount + this.weaponsCount + this.mountsCount;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(UnitGearManager.prototype, "weaponsCount", {
        get: function () { return this.weaponTypes.length; },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(UnitGearManager.prototype, "armorCount", {
        get: function () { return this.armorTypes.length; },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(UnitGearManager.prototype, "mountsCount", {
        get: function () { return this.mountTypes.length; },
        enumerable: false,
        configurable: true
    });
    return UnitGearManager;
}(AbstractManager));
export { UnitGearManager };
//# sourceMappingURL=unit-gear-manager.js.map