var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (Object.prototype.hasOwnProperty.call(b, p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        if (typeof b !== "function" && b !== null)
            throw new TypeError("Class extends value " + String(b) + " is not a constructor or null");
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
// common
import { Size, WorldPos } from '../world/pos';
import { GameOptions } from '../game-options';
// events
import { BaseEventListener } from '../../common/events';
import * as EV from "../game-events";
// world
import { WorldMap } from '../world/map';
import { WorldTile } from '../world/tile/tile';
import { TerrainTypes } from '../world/tile/tile-terrain';
import { TargetCursor } from '../world/cursor';
// entities
import { Player } from '../entities/player/player';
import { UnitArmy } from '../entities/unit/army/army';
// helpers
import { CityGenerator, CityLoader } from '../managers/world-loaders';
import { WorldTools } from '../../helpers/world-tools';
// babylon
import * as BABYLON from 'babylonjs';
var WorldMapScene = /** @class */ (function (_super) {
    __extends(WorldMapScene, _super);
    function WorldMapScene(canvasElement) {
        var _this = _super.call(this, "WorldScene") || this;
        // assets
        _this._blueprints = {};
        _this.tileInstances = [];
        _this.featureMeshes = [];
        _this.armyMeshes = [];
        // flags
        _this.assetsLoaded = false;
        // Create canvas and engine.
        _this._canvas = document.getElementById(canvasElement);
        _this.engine = new BABYLON.Engine(_this._canvas, true);
        return _this;
    }
    WorldMapScene.prototype.make = function () {
        // make tilemap model
        this.createMapObject();
        // create a basic BJS Scene object.
        globalThis.SCENE = new BABYLON.Scene(this.engine);
        this.delegateInput();
        // create materials & load models
        // this.startAssetLoading();
        globalThis.MAP.createMaterials();
        this.createWorld();
    };
    WorldMapScene.prototype.createWorld = function () {
        // create camera
        this.placePlayer();
        this.createPlayerCam();
        // create sun
        var sunId = WorldMapScene.NODE_ID_SUN;
        this._light = new BABYLON.HemisphericLight(sunId, new BABYLON.Vector3(0, 1, 0), globalThis.SCENE);
        // init tiles
        this.makeTileBlueprints(TerrainTypes.LIST);
        this.loadOrGenerateLocations();
        // show map
        // this.showMap(globalThis.MAP);
    };
    WorldMapScene.prototype.createPlayerCam = function () {
        var player = globalThis.PLAYER;
        if (!player) {
            this.error("cant create camera; player not set");
        }
        else {
            this.log("camera follows player at ".concat(player.pos));
        }
        globalThis.CAMERA = new BABYLON.ArcRotateCamera(WorldMapScene.NODE_ID_CAMERA, -Math.PI / 2, Math.PI / 4, 15, BABYLON.Vector3.Zero(), globalThis.SCENE);
        this.renewCameraTarget(player.mesh);
        globalThis.CAMERA.attachControl(this._canvas, true);
    };
    WorldMapScene.prototype.delegateInput = function () {
        globalThis.SCENE.onKeyboardObservable.add(function (kbInfo) {
            switch (kbInfo.type) {
                case BABYLON.KeyboardEventTypes.KEYDOWN:
                    // console.log("KEY DOWN: ", kbInfo.event.key);
                    break;
                case BABYLON.KeyboardEventTypes.KEYUP:
                    globalThis.INPUT.onKeyUp(kbInfo.event.key);
            }
        });
    };
    WorldMapScene.prototype.placePlayer = function () {
        this.log("creating player ..");
        globalThis.PLAYER = new Player();
    };
    WorldMapScene.prototype.makeTileBlueprints = function (terrainTypes) {
        this.log("making blueprints ..");
        for (var _i = 0, terrainTypes_1 = terrainTypes; _i < terrainTypes_1.length; _i++) {
            var terrainId = terrainTypes_1[_i];
            this._blueprints[terrainId] = this.makeTileBlueprint(terrainId);
        }
    };
    /* creates Babylon hexagon Mesh for every terrain tile, that will be used
       to make mesh instances of */
    WorldMapScene.prototype.makeTileBlueprint = function (terrainType) {
        var diameterBottom = WorldTile.HEXSIZE;
        var tileName = "base-tile-".concat(terrainType);
        var mesh = BABYLON.Mesh.CreateCylinder(tileName, TerrainTypes.heightOf(terrainType), TerrainTypes.topDiameterOf(terrainType), diameterBottom, 6, // tesselation
        1, // subdivision
        globalThis.SCENE, false // updatable
        );
        mesh.rotation.y = Math.PI / 2;
        mesh.position.y = TerrainTypes.heightOf(terrainType) / 2;
        mesh.convertToFlatShadedMesh();
        mesh.layerMask = 0;
        mesh.material = this.getMaterial(terrainType);
        var viewDist = globalThis.OPTIONS.getNum(GameOptions.MAP_VIEW_DISTANCE);
        mesh.addLODLevel(viewDist, null);
        if (mesh.material) {
            mesh.material.freeze();
        }
        mesh.freezeWorldMatrix();
        return mesh;
    };
    WorldMapScene.prototype.getMaterial = function (key) {
        return globalThis.MAP.mats.get(key);
    };
    WorldMapScene.prototype.renewCameraTarget = function (mesh) {
        globalThis.CAMERA.setTarget(mesh);
        // turn to right angle 
        globalThis.CAMERA.radius = CAMERA_RADIUS;
        globalThis.CAMERA.alpha = CAMERA_ALPHA;
        globalThis.CAMERA.beta = CAMERA_BETA;
    };
    WorldMapScene.prototype.createMapObject = function () {
        // get map name and default size
        var mapName = globalThis.MAP_NAME;
        var worldSize = new Size(100, 100);
        this.log("creating map ".concat(mapName));
        // generate and return WorldMap model
        globalThis.MAP = new WorldMap(mapName, worldSize);
        console.log(globalThis.MAP);
        return globalThis.MAP;
    };
    WorldMapScene.prototype.placeNewMap = function (worldMap) {
        this.log("showing map");
        this.placeMapTiles(worldMap);
        this.placeTargetCursor();
        // loading citied & other locations
        this.placeLocations();
        // place landcape features
        this.placeLandscapeFeatures();
        // place armies & traders
        this.placeOutsideArmies();
        // replace player mesh
        globalThis.PLAYER.updateMesh();
        this.renewCameraTarget(globalThis.PLAYER.mesh);
    };
    /* PLACE MAP TILES */
    WorldMapScene.prototype.placeMapTiles = function (worldMap) {
        for (var y = 0; y < worldMap.size.z; y++) {
            for (var x = 0; x < worldMap.size.x; x++) {
                var tile = worldMap.tileAt(x, y);
                this.placeSingleMapTile(tile);
            }
        }
        this.log("placed ".concat(this.tileInstances.length, " MapTiles"));
    };
    WorldMapScene.prototype.placeSingleMapTile = function (tile) {
        if (!tile) {
            return;
        }
        // offset every second row
        var offset = (tile.pos.z % 2 === 0) ? WorldTile.HEXSIZE / 2 : 0;
        // show tile mesh
        var tileInstance = this.createTileInstance(tile, offset);
        this.tileInstances.push(tileInstance);
        return tileInstance;
    };
    WorldMapScene.prototype.getBlueprint = function (terrainChar) {
        if (!terrainChar) {
            terrainChar = TerrainTypes.UNKNOWN;
        }
        var blueprint = this._blueprints[terrainChar];
        if (!blueprint) {
            this.error("cant create tile with terrain ".concat(terrainChar, " in ").concat(this._blueprints));
            return;
        }
        return blueprint;
    };
    WorldMapScene.prototype.createTileInstance = function (tile, offset) {
        var blueprint = this.getBlueprint(tile.terrainChar);
        if (!blueprint) {
            this.warn("cant create tile instance of \"".concat(tile.terrainChar, "\" at ").concat(tile.pos));
            return;
        }
        // create new instance from terrain blueprint
        var meshInstance = blueprint.createInstance(tile.meshName);
        meshInstance.position = WorldTools.worldToScenePos(tile.pos);
        // link mesh to tile model
        tile.meshInstance = meshInstance;
        tile.offset = offset;
        return meshInstance;
    };
    Object.defineProperty(WorldMapScene.prototype, "topleft", {
        get: function () {
            var map = globalThis.MAP;
            return new WorldPos(-map.size.x * WorldTile.HEXSIZE / 2, map.size.z * WorldTile.HEXSIZE / 2);
        },
        enumerable: false,
        configurable: true
    });
    WorldMapScene.prototype.getOffsetAt = function (z) {
        return (z % 2 === 0) ? WorldTile.HEXSIZE / 2 : 0;
    };
    WorldMapScene.prototype.loadOrGenerateLocations = function () {
        this.loadOrGenerateCities();
    };
    WorldMapScene.prototype.placeLandscapeFeatures = function () {
        var mapObj = globalThis.MAP;
        var placedFeatureCount = 0;
        for (var _i = 0, _a = mapObj.features; _i < _a.length; _i++) {
            var feature = _a[_i];
            var successfullyPlaced = this._placeSingleLandscapeFeature(feature);
            if (successfullyPlaced) {
                placedFeatureCount += 1;
            }
        }
        this.log("placed ".concat(placedFeatureCount, " landscape features on map"));
    };
    WorldMapScene.prototype._placeSingleLandscapeFeature = function (feature) {
        var asset = globalThis.ASSETS.getAsset(feature.id);
        if (!asset) {
            this.warn("cant place landscape feature \"".concat(feature.id, "\"; asst not found"));
            return false;
        }
        var meshId = "lds-feature-".concat(this.featureMeshes.length);
        var meshInstance = WorldTools.placeAssetInstance(feature.id, meshId, feature.pos, true, true);
        this.featureMeshes.push(meshInstance);
        return meshInstance != null;
    };
    WorldMapScene.prototype.placeLocations = function () {
        // this.loadOrGenerateCities();
        this.placeCities();
    };
    WorldMapScene.prototype.loadOrGenerateCities = function () {
        // get loaed or generated City array
        var cities = this.getLoadedOrGeneratedCities();
        // add new cities to CityList holder & inform CityManager
        for (var _i = 0, cities_1 = cities; _i < cities_1.length; _i++) {
            var city = cities_1[_i];
            globalThis.CITIES.addCity(city);
        }
        globalThis.CITIES.onCitiesLoaded();
    };
    WorldMapScene.prototype.getLoadedOrGeneratedCities = function () {
        var generateLocations = globalThis.OPTIONS.getBool(GameOptions.GENERATE_LOCATIONS);
        // generate or load cities
        if (generateLocations) {
            return this.generateRandomCities();
        }
        return this.loadWorldCities();
    };
    WorldMapScene.prototype.generateRandomCities = function () {
        this.log("generating cities ..");
        var generatedCities = new CityGenerator().generate();
        this.log("generated ".concat(generatedCities.length, " cities"));
        return generatedCities;
    };
    WorldMapScene.prototype.loadWorldCities = function () {
        this.log("loading cities ..");
        var loadedCities = CityLoader.load();
        this.log("loaded ".concat(loadedCities.length, " cities starting with ").concat(loadedCities[0].name));
        return loadedCities;
    };
    WorldMapScene.prototype.placeCities = function () {
        var cities = globalThis.CITIES.cities;
        var placedCitiesCount = 0;
        this.log("placing ".concat(cities.length, " cities starting with ").concat(cities[0].name, " .."));
        for (var _i = 0, cities_2 = cities; _i < cities_2.length; _i++) {
            var city = cities_2[_i];
            this.placeSingleCity(city);
            this.showCityConnections(city);
            placedCitiesCount += 1;
        }
        // fire CitiesPlayedEvent
        globalThis.EVENTS.fireEvent(new EV.CitiesPlacedEvent(placedCitiesCount));
        // globalThis.CITIES.onCityMeshesPlaced();
    };
    WorldMapScene.prototype.checkAllCitiesPlaces = function () {
    };
    WorldMapScene.prototype.placeSingleCity = function (city) {
        var cityAssetName = city.getAssetName();
        city.mesh = globalThis.ASSETS.getAsset(cityAssetName).getMeshInstance(city.meshId, true);
        WorldTools.placeMesh(city.mesh, city.pos, true, true);
    };
    WorldMapScene.prototype.showCityConnections = function (city) {
        if (!SHOW_CONNECTION_LINES) {
            return;
        }
        var sourceVec3 = WorldTools.worldToScenePos(city.pos);
        sourceVec3.y = 0.9;
        var targetVec3Arr = [];
        for (var _i = 0, _a = city.getConnectedTradeLocations(); _i < _a.length; _i++) {
            var connectedCities = _a[_i];
            var targetPos = WorldTools.worldToScenePos(connectedCities.pos);
            targetPos.y = .9;
            targetVec3Arr.push(targetPos);
        }
        var connectionNum = 0;
        for (var _b = 0, targetVec3Arr_1 = targetVec3Arr; _b < targetVec3Arr_1.length; _b++) {
            var targetVec3 = targetVec3Arr_1[_b];
            var lineOptions = { points: [sourceVec3, targetVec3] };
            // create line
            var connectionLineId = "conn-" + city.key + "-" + connectionNum;
            var line = BABYLON.MeshBuilder.CreateLines(connectionLineId, lineOptions, globalThis.SCENE);
            line.alpha = 0.3;
            // line.edgesRenderer = new BABYLON.LineEdgesRenderer(line);
            line.edgesWidth = 3;
            line.freezeWorldMatrix();
            connectionNum += 1;
        }
    };
    /* == ARMY PLACEMENT == */
    WorldMapScene.prototype.placeOutsideArmies = function () {
        var outsideArmies = globalThis.ARMIES.getOutsideArmies();
        for (var _i = 0, outsideArmies_1 = outsideArmies; _i < outsideArmies_1.length; _i++) {
            var army = outsideArmies_1[_i];
            this.placeSingleArmy(army);
        }
    };
    WorldMapScene.prototype.placeSingleArmy = function (army) {
        var assetName = UnitArmy.DEFAULT_MODEL;
        var meshId = "army-" + army.num;
        var instance = WorldTools.placeAssetInstance(assetName, meshId, army.pos, true, true);
        // add to army meshes holder
        this.armyMeshes.push(instance);
    };
    WorldMapScene.prototype.placeTargetCursor = function () {
        var cursorDefaultAssetName = TargetCursor.DEFAULT_ASSET_NAME;
        var cursorStartPos = new WorldPos(20, 20);
        this.cursorMesh = WorldTools.placeAssetInstance(cursorDefaultAssetName, TargetCursor.NODE_ID, cursorStartPos, true, true);
    };
    WorldMapScene.prototype.onAllAssetsLoaded = function () {
        this.placeNewMap(globalThis.MAP);
    };
    WorldMapScene.prototype.doRender = function () {
        var _this = this;
        // Run the render loop.
        this.engine.runRenderLoop(function () {
            globalThis.SCENE.render();
        });
        // The canvas/window resize event handler.
        window.addEventListener('resize', function () {
            _this.log("resized");
            _this.engine.resize();
        });
    };
    Object.defineProperty(WorldMapScene, "NODE_ID_CAMERA", {
        get: function () { return "cam-main"; },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(WorldMapScene, "NODE_ID_SUN", {
        get: function () { return "sun"; },
        enumerable: false,
        configurable: true
    });
    WorldMapScene.prototype.toString = function () {
        var mapName = "no-map";
        var mapSize = "no-size";
        if (globalThis.MAP) {
            mapName = globalThis.MAP.name;
            mapSize = "".concat(globalThis.MAP.tiles[0].length, "x").concat(globalThis.MAP.tiles.length);
        }
        return "<WorldScene \"".concat(mapName, "\" tiles=").concat(mapSize, "> ");
    };
    // events
    WorldMapScene.prototype.onEvent = function (ev) {
        _super.prototype.onEvent.call(this, ev);
    };
    return WorldMapScene;
}(BaseEventListener));
export default WorldMapScene;
/* == SCENE CONSTANTS == */
export var SHOW_CITY_MODELS = true;
// camera
export var CAMERA_RADIUS = 25;
export var CAMERA_ALPHA = -1 * Math.PI / 2;
export var CAMERA_BETA = 0.84;
export var SHOW_CONNECTION_LINES = false;
//# sourceMappingURL=world-scene.js.map