var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (Object.prototype.hasOwnProperty.call(b, p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        if (typeof b !== "function" && b !== null)
            throw new TypeError("Class extends value " + String(b) + " is not a constructor or null");
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
// helpers
import { RandomInt } from "../../helpers/random";
import { WorldTools } from "../../helpers/world-tools";
// babylon 
import * as BABYLON from 'babylonjs';
/* SIZE */
var Size = /** @class */ (function () {
    function Size(x, z) {
        this.x = x;
        this.z = z;
    }
    return Size;
}());
export { Size };
/* POS TYPE ENUM */
var PosTypes = /** @class */ (function () {
    function PosTypes() {
    }
    Object.defineProperty(PosTypes, "ABSTRACT", {
        get: function () { return "abstract"; },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(PosTypes, "WORLD", {
        get: function () { return "world"; },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(PosTypes, "WORLD_REL", {
        get: function () { return "rel-world"; },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(PosTypes, "LOCATION", {
        get: function () { return "location"; },
        enumerable: false,
        configurable: true
    });
    return PosTypes;
}());
export { PosTypes };
var AbstractPos = /** @class */ (function () {
    function AbstractPos(x, z, parseInts) {
        this.x = -1;
        this.z = -1;
        this.type = PosTypes.ABSTRACT;
        this.extraData = null;
        if (parseInts) {
            this.x = Math.round(x);
            this.z = Math.round(z);
        }
        else {
            this.x = x;
            this.z = z;
        }
    }
    return AbstractPos;
}());
export { AbstractPos };
var WorldPos = /** @class */ (function (_super) {
    __extends(WorldPos, _super);
    function WorldPos() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.type = PosTypes.WORLD;
        return _this;
    }
    WorldPos.random = function () {
        if (!globalThis.MAP) {
            console.error("WorldPos: cant generate random; map not loaded");
            return null;
        }
        var mapSize = globalThis.MAP.size;
        var rndXZ = [RandomInt.below(mapSize.x), RandomInt.below(mapSize.z)];
        return new WorldPos(rndXZ[0], rndXZ[1]);
    };
    WorldPos.prototype.inDirection = function (dir) {
        var relPos = dir.relPos;
        var xz = [this.x + relPos.x, this.z + relPos.z];
        return new WorldPos(xz[0], xz[1]);
    };
    WorldPos.fromXZArray = function (xz) {
        return new WorldPos(xz[0], xz[1]);
    };
    WorldPos.FromString = function (str, pipeDivider) {
        var dividerChar = " ";
        if (pipeDivider) {
            dividerChar = "|";
        }
        // get x & z position values from string
        var asArray = str.split(dividerChar);
        var xz = [parseInt(asArray[0]), parseInt(asArray[1])];
        var pos = WorldPos.fromXZArray(xz);
        // add extra info str?
        if (asArray.length > 2) {
            pos.extraData = asArray[2];
        }
        return pos;
    };
    WorldPos.prototype.equals = function (otherPos) {
        return this.x == otherPos.x && this.z == otherPos.z;
    };
    WorldPos.prototype.asString = function () {
        var posDivider = "|";
        return this.x + posDivider + this.z;
    };
    WorldPos.prototype.withRelPos = function (relPos) {
        var absPosNumArr = [this.x + relPos.x, this.z + relPos.z];
        return WorldPos.fromXZArray(absPosNumArr);
    };
    /* returns the sreen x y coordianes of a world position: currently doesnt works tho */
    WorldPos.prototype.toWindowPos = function () {
        var coords = BABYLON.Vector3.Project(WorldTools.worldToScenePos(this), BABYLON.Matrix.Identity(), globalThis.SCENE.getTransformMatrix(), globalThis.CAMERA.viewport.toGlobal(globalThis.WORLD.engine.getRenderWidth(), globalThis.WORLD.engine.getRenderHeight()));
        return null;
    };
    /* returns whether a pos has an uneven z value and is with that moved half a tile space to the right */
    WorldPos.prototype.hasUnevenZ = function () {
        return this.z % 2 == 1;
    };
    Object.defineProperty(WorldPos.prototype, "neighborRelPositions", {
        get: function () {
            var neighborRelPositions = [new WorldPos(-1, 0), new WorldPos(1, 0)];
            var relStartX = 0;
            if (this.hasUnevenZ) {
                relStartX = -1;
            }
            // add two pos to the top
            neighborRelPositions.concat([new WorldRelPos(relStartX, -1), new WorldRelPos(relStartX + 1, -1)]);
            // .. and to the bottom of current pos
            neighborRelPositions.concat([new WorldRelPos(relStartX, 1), new WorldRelPos(relStartX + 1, 1)]);
            return neighborRelPositions;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(WorldPos.prototype, "neighborAbsPositions", {
        get: function () {
            var neighborAbsPositions = [];
            for (var _i = 0, _a = this.neighborRelPositions; _i < _a.length; _i++) {
                var neighborRelPos = _a[_i];
                var absPos = this.withRelPos(neighborRelPos);
                neighborAbsPositions.push(absPos);
            }
            return neighborAbsPositions;
        },
        enumerable: false,
        configurable: true
    });
    WorldPos.prototype.distanceTo = function (otherPos) {
        var distX = Math.abs(this.x - otherPos.x);
        var distZ = Math.abs(this.z - otherPos.z);
        return distX + distZ;
    };
    WorldPos.prototype.closestPosToTarget = function (targetPos) {
        var dist = [this.x - targetPos.x, this.z - targetPos.z];
        var distAbs = [Math.abs(dist[0]), Math.abs(dist[1])];
        // x or z axis?
        var relXZ = [dist[0] / distAbs[0], 0];
        if (distAbs[1] > distAbs[0]) {
            relXZ = [0, dist[1] / distAbs[1]];
        }
        var nextPos = new WorldPos(this.x + relXZ[0], this.z + relXZ[1]);
        return nextPos;
    };
    return WorldPos;
}(AbstractPos));
export { WorldPos };
var WorldRelPos = /** @class */ (function (_super) {
    __extends(WorldRelPos, _super);
    function WorldRelPos(x, z, parseInts) {
        var _this = _super.call(this, x, z, parseInts) || this;
        _this.type = PosTypes.WORLD_REL;
        return _this;
    }
    return WorldRelPos;
}(WorldPos));
export { WorldRelPos };
var LocationPos = /** @class */ (function (_super) {
    __extends(LocationPos, _super);
    function LocationPos() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.type = PosTypes.LOCATION;
        return _this;
    }
    return LocationPos;
}(AbstractPos));
export { LocationPos };
/* DIRECTION */
var HexDirection = /** @class */ (function () {
    function HexDirection(dirStr) {
        this.dirStr = dirStr;
    }
    Object.defineProperty(HexDirection, "WEST", {
        get: function () { return "W"; },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(HexDirection, "EAST", {
        get: function () { return "E"; },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(HexDirection, "NORTHWEST", {
        get: function () { return "NW"; },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(HexDirection, "NORTHEAST", {
        get: function () { return "NE"; },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(HexDirection, "SOUTHWEST", {
        get: function () { return "SW"; },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(HexDirection, "SOUTHEAST", {
        get: function () { return "SE"; },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(HexDirection, "ALL", {
        get: function () {
            return [HexDirection.WEST, HexDirection.EAST, HexDirection.NORTHWEST,
                HexDirection.NORTHEAST, HexDirection.SOUTHWEST, HexDirection.SOUTHEAST];
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(HexDirection.prototype, "relPos", {
        get: function () {
            switch (this.dirStr) {
                case "W": return new WorldPos(-1, 0);
                case "E": return new WorldPos(1, 0);
                case "NW": return new WorldPos(-1, -1);
                case "NE": return new WorldPos(0, -1);
                case "SW": return new WorldPos(-1, 1);
                case "SE": return new WorldPos(1, 1);
                default: return new WorldPos(0, 0);
            }
        },
        enumerable: false,
        configurable: true
    });
    return HexDirection;
}());
export { HexDirection };
export var AXIS_X = 0;
export var AXIS_Z = 1;
//# sourceMappingURL=pos.js.map