var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (Object.prototype.hasOwnProperty.call(b, p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        if (typeof b !== "function" && b !== null)
            throw new TypeError("Class extends value " + String(b) + " is not a constructor or null");
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
// common
import { Loggable } from "../../common/log";
/* == WORLD SELECTION MODEL == */
var WorldSelection = /** @class */ (function (_super) {
    __extends(WorldSelection, _super);
    function WorldSelection(pos) {
        var _this = _super.call(this, "WorldSelection") || this;
        _this._lastPos = null;
        _this._pos = null;
        _this._pos = pos;
        // create selected entity holder
        _this.entity = new SelectedEntity(SELECTED_ENTITY_TYPE_NONE);
        _this.entity.parent = _this;
        return _this;
    }
    WorldSelection.prototype.setToPlayer = function () { this.entity.setToPlayer(); };
    WorldSelection.prototype.setToCity = function (city) { this.entity.setToCity(city); };
    WorldSelection.prototype.setToArmy = function (army) { this.entity.setToArmy(army); };
    WorldSelection.prototype.setToNone = function () { this.entity.setToNone(); };
    WorldSelection.prototype.updatePos = function (newPos) {
        var oldPos = this._pos;
        this._lastPos = oldPos;
        this._pos = newPos;
    };
    return WorldSelection;
}(Loggable));
export { WorldSelection };
var SelectedEntity = /** @class */ (function (_super) {
    __extends(SelectedEntity, _super);
    function SelectedEntity(entityTypeNum, entityKey) {
        var _this = _super.call(this, "SelectedEntity") || this;
        _this.entityTypeNum = SELECTED_ENTITY_TYPE_NONE;
        _this.entityTypeNum = entityTypeNum;
        if (entityTypeNum == SELECTED_ENTITY_TYPE_CITY && entityKey) {
            _this.setToCityByKey(entityKey);
        }
        return _this;
    }
    SelectedEntity.prototype.setToNone = function () { this.unset(); };
    /* SET TO PLAYER, CITY OR ARMY */
    SelectedEntity.prototype.setToPlayer = function () {
        this.entityTypeNum = SELECTED_ENTITY_TYPE_PLAYER;
        this.entityKey = null;
        // unset entity but update pos
        this._entity = null;
        this.updateParentPos();
    };
    SelectedEntity.prototype.setToCity = function (selectedCity) {
        // update selected entity type & key
        this.entityTypeNum = SELECTED_ENTITY_TYPE_CITY;
        this.entityKey = selectedCity.key;
        // and model
        this._setToEntity(selectedCity);
    };
    SelectedEntity.prototype._setToEntity = function (entity) {
        this._entity = entity;
        this.updateParentPos();
    };
    SelectedEntity.prototype.unset = function () {
        this.entityTypeNum = SELECTED_ENTITY_TYPE_NONE;
        this._entity = null;
        this.parent.updatePos(null);
    };
    SelectedEntity.prototype.updateParentPos = function () {
        if (this.entity && this.parent) {
            this.parent.updatePos(this.entity.pos);
        }
    };
    SelectedEntity.prototype.setToCityByKey = function (selectedCityKey) {
        if (!globalThis.CITIES) {
            this.log("cant update selection with city \"".concat(selectedCityKey, "\": CityManager not found"));
            return;
        }
        var city = globalThis.CITIES.getByKey(selectedCityKey);
        this.setToCity(city);
    };
    SelectedEntity.prototype.setToArmy = function (selectedArmy) {
        this.entityTypeNum = SELECTED_ENTITY_TYPE_ARMY;
        this.entityKey = "" + selectedArmy.num;
        this._setToEntity(selectedArmy);
    };
    Object.defineProperty(SelectedEntity.prototype, "entity", {
        /* SELECTED ENTITY GETTERS */
        get: function () {
            if (this.isNone) {
                return;
            }
            // is player?
            if (this.isPlayer) {
                return globalThis.PLAYER;
            }
            // is city or army?
            if (this.isCity && this._entity) {
                return this._entity;
            }
            if (this.isArmy && this._entity) {
                return this._entity;
            }
            // throw warning if not
            this.warn("cant find entity of type ".concat(this.entityTypeName));
            return null;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(SelectedEntity.prototype, "entityPos", {
        get: function () {
            if (this.entity) {
                return this.entity.pos;
            }
            return;
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(SelectedEntity.prototype, "entityTypeName", {
        get: function () {
            return SELECTED_ENTITY_TYPE_NAMES[this.entityTypeNum];
        },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(SelectedEntity.prototype, "isNone", {
        /* SELECTED ENTITY FLAGS */
        get: function () { return this.entityTypeNum == SELECTED_ENTITY_TYPE_NONE; },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(SelectedEntity.prototype, "isPlayer", {
        get: function () { return this.entityTypeNum == SELECTED_ENTITY_TYPE_PLAYER; },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(SelectedEntity.prototype, "isCity", {
        get: function () { return this.entityTypeNum == SELECTED_ENTITY_TYPE_CITY; },
        enumerable: false,
        configurable: true
    });
    Object.defineProperty(SelectedEntity.prototype, "isArmy", {
        get: function () { return this.entityTypeNum == SELECTED_ENTITY_TYPE_ARMY; },
        enumerable: false,
        configurable: true
    });
    return SelectedEntity;
}(Loggable));
export { SelectedEntity };
/* == SELECTION CONSTANTS == */
export var SELECTED_ENTITY_TYPE_NONE = 0;
export var SELECTED_ENTITY_TYPE_PLAYER = 1;
export var SELECTED_ENTITY_TYPE_CITY = 2;
export var SELECTED_ENTITY_TYPE_ARMY = 3;
export var SELECTED_ENTITY_TYPE_NAMES = ["None", "Player", "City", "Army"];
//# sourceMappingURL=selection.js.map