// common
import { GameOptions } from '../game/game-options';
import { RandomRotation } from './random';
import { WorldTile } from '../game/world/tile/tile';
import { TerrainTypes } from '../game/world/tile/tile-terrain';
// babylon
import * as BABYLON from 'babylonjs';
var WorldTools = /** @class */ (function () {
    function WorldTools() {
    }
    WorldTools.setMeshScenePos = function (mesh, worldPos) {
        var scenePos = WorldTools.worldToScenePos(worldPos);
        mesh.position = scenePos;
        // add terrain offset
        var terrainChar = globalThis.MAP.tileAtPos(worldPos).terrainChar;
        if (terrainChar) {
            var yOffset = TerrainTypes.heightOf(terrainChar) / 2;
            mesh.position.y += yOffset;
        }
    };
    WorldTools.setMeshLOD = function (mesh) {
        if (mesh instanceof BABYLON.Mesh) {
            var fullMesh = mesh;
            var viewDist = globalThis.OPTIONS.getNum(GameOptions.MAP_VIEW_DISTANCE);
            fullMesh.addLODLevel(viewDist, null);
            return fullMesh;
        }
        return mesh;
    };
    WorldTools.setMeshScaling = function (mesh, scale) {
        mesh.scaling.set(scale, scale, scale);
    };
    WorldTools.placeMesh = function (mesh, worldPos, autoscale, autorotate) {
        if (!mesh) {
            console.warn("cant place mesh ".concat(mesh.name));
            return;
        }
        WorldTools.setMeshScenePos(mesh, worldPos);
        WorldTools.setMeshLOD(mesh);
        if (autoscale) {
            WorldTools.setMeshScaling(mesh, 2);
        }
        if (autorotate) {
            RandomRotation.onMesh(mesh);
        }
        return mesh;
    };
    WorldTools.placeAssetInstance = function (assetName, instanceMehsId, worldPos, autoscale, autorotate) {
        var assetInstance = null;
        if (globalThis.ASSETS) {
            assetInstance = globalThis.ASSETS.getAsset(assetName).getMeshInstance(instanceMehsId);
        }
        if (!assetInstance) {
            return;
        }
        return WorldTools.placeMesh(assetInstance, worldPos, autoscale, autorotate);
    };
    WorldTools.getMeshDistance = function (mesh1, mesh2) {
        if (!mesh1 && mesh2) {
            return;
        }
        return WorldTools.getVec3Distance(mesh1.position, mesh2.position);
    };
    WorldTools.getVec3Distance = function (source, target) {
        return BABYLON.Vector3.Distance(source, target);
    };
    WorldTools.getWorldPosDistance = function (pos1, pos2) {
        var scenePos1 = WorldTools.worldToScenePos(pos1);
        var scenePos2 = WorldTools.worldToScenePos(pos2);
        return WorldTools.getVec3Distance(scenePos1, scenePos2);
    };
    WorldTools.worldToScenePos = function (worldPos) {
        if (!globalThis.WORLD) {
            return;
        }
        var hexSize = WorldTile.HEXSIZE;
        var worldTopLeft = globalThis.WORLD.topleft;
        var xOffset = globalThis.WORLD.getOffsetAt(worldPos.z);
        var x = (worldTopLeft.x + worldPos.x * hexSize + xOffset) * 0.9;
        var z = (worldTopLeft.z - worldPos.z * hexSize) * 0.8;
        return new BABYLON.Vector3(x, 0, z);
    };
    return WorldTools;
}());
export { WorldTools };
//# sourceMappingURL=world-tools.js.map