// trade
import { StoredResource } from '../trade/resource/resource-storage';

/* == INTERFACES == */
export interface IPopulationClass {
    // basic properties
    level: number;
    name: string;
    taxRate: number;
    iconName: string;
    // consumtion & upgrade resources
    consumesResources: Array<StoredResource>;
    upgradeResource: StoredResource;
    // methods
    getConsumedResIds(): Array<string>;
    getResourceConsumption(resId): number;
}

/* SINGLE POPULATION CLASS */
export class SinglePopulationClass implements IPopulationClass {
    // basic properties
    level: number;
    name: string;
    taxRate: number = .2;
    iconName: string = "trader";
    // consumption & upgrade resources
    consumesResources: Array<StoredResource> = [];
    upgradeResource: StoredResource = null;

    constructor(level: number, name: string, iconName: string, consumesResourcesStr: string, taxes?: number, upgradeResStr?: string) {
        this.level = level;
        this.name = name;
        if ( taxes ) { this.taxRate = taxes; }
        // icon
        this.iconName = iconName;
        this.setConsumedResourcesFromString(consumesResourcesStr);
    }

    /* consumption is for every 10 pops, so it will usually by lower than the given amount */
    private setConsumedResourcesFromString(consumesResourcesStr) {
        const splitted = consumesResourcesStr.split(" ");
        for (const storedResStr of splitted) {
            const storedRes = StoredResource.fromString(storedResStr);
            this.consumesResources.push(storedRes);
        }
    }

    private setUpdateResourceFromString(upgradeResStr) {
        const upgradeRes = StoredResource.fromString(upgradeResStr);
        this.upgradeResource = upgradeRes;
    }

    getConsumedResIds(): Array<string> {
        let consumedResIds: Array<string> = [];
        for (const consumedRes of this.consumesResources) {
            consumedResIds.push(consumedRes.resId);
        }
        return consumedResIds;
    }

    getResourceConsumption(resId): number {
        for (const consumedRes of this.consumesResources) {
            if (consumedRes.resId == resId) { return consumedRes.amount; }
        }
        return 0;
    }
}

/* CONSTANTS */
// popuulation class ids
export const POP_BEGGARS_NUM = 0;
export const POP_PEASANTS_NUM = 1;
export const POP_CITIZEN_NUM = 2;
export const POP_CLERICAL_NUM = 3;
export const POP_MERCHANTS_NUM = 4;
export const POP_NOBLES_NUM = 5;

// SinglePopulationClass object definitions
export const POP_CLASSES = [
    new SinglePopulationClass(POP_BEGGARS_NUM, "Beggars", "beggar", "0.3xale 0.3xfish 1xwood", .5, "0.5xtools"),
    new SinglePopulationClass(POP_PEASANTS_NUM, "Peasants", "peasant", "0.4xgrain 0.3xfish 0.2xale", 2, "0.4xtools"),
    new SinglePopulationClass(POP_CITIZEN_NUM, "Citizen", "citizen", "0.5xgrain 0.3xpelts 0.2xale", 1.5, "0.2xclothes"),
    new SinglePopulationClass(POP_CLERICAL_NUM, "Clerical", "clerical", "0.4xale 0.2xclothes 0.1xpearls", 2, "0.2xgold"),
    new SinglePopulationClass(POP_MERCHANTS_NUM, "Merchants", "trader", "0.7xgrain 0.2xclothes 0.2xtools 0.2xpearls", 3.3, "0.4xgold"),
    new SinglePopulationClass(POP_NOBLES_NUM, "Nobles", "noble", "1xgrain 0.3xgold 0.5xclothes 0.1xweapons", 4)
];
