import { WorldTile } from "../../../world/tile/tile";
import { WorldPos } from "../../../world/pos";
import { City } from "../../city/city";
import { IConnection } from "./connection";
import { IConnectedLocation } from "../../../world/location";

/* == CONNECTION PATH INTERFACE == */
export interface IConnectionPath {
    parent: IConnection;
    start: WorldPos;
    target: WorldPos;
    path: WorldPos[];
}

/* == CONNECTION PATH MODEL == */
export class AbstractConnectionPath implements IConnectionPath {
    parent: IConnection;
    start: WorldPos;
    target: WorldPos;
    path: WorldPos[] = [];

    constructor(parent: IConnection, start: WorldPos, target: WorldPos) {
        this.parent = parent;
        this.start = start;
        this.target = target;
    }

    setDirectPath() {
        // add start pos
        this.addNextPos(this.start, true);
        let length: number = 0;
        let notAtTarget = ! this.isAtTarget();
        while(! this.isAtTarget()) {
            let nextPos: WorldPos = this.getNextPos();
            this.addNextPos(nextPos, true);
            length += 1;
        }
    }

    /* adds WorldPosition into array of positions to target */
    addNextPos(pos: WorldPos, addToTile?: boolean) {
        this.path.push(pos);
        if (addToTile == true) {
            const tile = globalThis.MAP.tileAtPos(pos);
            tile.connections.push(this.parent);
        }
    }

    /* finds next WorldPos towards target */
    getNextPos(): WorldPos {
        let lastPos: WorldPos = this.getLastPos();
        let relDist = [this.target.x - lastPos.x, this.target.z - lastPos.z];
        // get relative position towards next position (example: [-1, 0] = one to the west)
        let relXZ = [relDist[0] / Math.abs(relDist[0]), relDist[1] / Math.abs(relDist[1])];
        // start from absolute pos ..
        let absX: number = lastPos.x;
        let absZ: number = lastPos.z;
        // .. and move horizontally ..
        if ( relXZ[0] ) { absX = lastPos.x + relXZ[0]; } 
        // .. or vertically
        if ( relXZ[1] ) { absZ = lastPos.z + relXZ[1]; }
        return new WorldPos(absX, absZ);
    }

    getLastPos(): WorldPos {
        return this.path[this.path.length - 1];
    }

    isAtTarget(): boolean {
        let lastPos = this.getLastPos();
        let isAtTarget: boolean = (lastPos.x == this.target.x) && (lastPos.z == this.target.z);
        return isAtTarget;
    }
}

/* == CITY CONN PATH MODEL == */
export class LocationConnectionPath extends AbstractConnectionPath implements AbstractConnectionPath {
    startLocation: IConnectedLocation;
    targetLocation: IConnectedLocation;
    path: WorldPos[] = [];

    constructor(parent: IConnection, startLocation: IConnectedLocation, targetLocation: IConnectedLocation) {
        super(parent, startLocation.pos, targetLocation.pos);
        this.startLocation = startLocation;
        this.targetLocation = targetLocation;
    }

}