// common
import { Loggable } from "../../../common/log";
// entities
import { CommandableMapEntity, IUniqueInstance } from "../../../common/entity";
// world
import { WorldPos } from "../../world/pos";
// entities
import { UnitArmy } from "./army/army";
import { IUnitType, UnitType } from "./type/unit-type";

/* == INTERFACES == */
export interface IUnitInstance extends IUniqueInstance  {
    num: number;
    type: IUnitType;
    pos: WorldPos;
    hp: number;
    mp: number;
}

export interface IUnitMover {
    maxMovement: number;
    currentMovement: number;
    canMoveInDir(dirStr: string);
    moveInDir(dirStr: string, force?: boolean);
}

/* == UNIT MODEL == */
export class Unit extends CommandableMapEntity implements IUnitInstance {
    num: number;
    type: IUnitType;
    pos: WorldPos;
    hp: number;
    mp: number;
    army: UnitArmy = null;

    constructor(typeId: string, pos: WorldPos, army?: UnitArmy) {
        super(globalThis.UNITS.nextUnitId(), pos);
        // this.num = globalThis.UNITS.nextUnitId();
        this.type = globalThis.UNIT_TYPES.get(typeId);
        if (army) {
            this.army = army;
        }
        this.hp = this.type.health;
        this.mp = this.type.movement;
    }

    toString(): string {
        let unitPos: WorldPos = this.pos;
        if (! unitPos ) {
            unitPos = this.army.pos;
        }
        return `<Unit ${this.num} "${this.type.id}" pos=${unitPos.asString()} hp=${this.hp} mp=${this.mp}>`;
    }
}

/* == UNIT MOVEMENT == */