// city & buildings
import { IBuildingType, ProductionBuildingType } from "../../entities/building/type/building-type";
import { City } from "../../entities/city/city";
import { BuildingCategory } from "../../entities/building/building-category";
// gui
import * as GUI_CONST from '../common/constants';
import { EmptyDialog, IDialog } from "./common/base-dialog";
import { GuiScrollView } from "../elements/scroller";
// babylon
import * as GUI from 'babylonjs-gui';
import {IConstructable} from "../../entities/building/type/constructable";

/* == CITY CONSTRUCTION DIALOG == */
export class CityConstructionDialog extends EmptyDialog {
    city: City;
    // chil elements
    scrollViewer: GUI.ScrollViewer;
    innerLayout: GUI.StackPanel;

    constructor(city: City) {
        super(DIAG_CONSTR, false, `${city.name} Construction`);
        this.city = city;
    }

    addCustomContent(): void {
        const switcherOptionStrs = DIAG_CONSTR_SWITCHER_OPTIONS_NAMES;
        this.addNewSwitcher(DIAG_CONSTR_SWITCHER, switcherOptionStrs);
        // inner layout
        this.createInnerLayout();
        // switcher
        const startOption = DIAG_CONSTR_SWITCHER_OPTIONS_NAMES[0];
        this.onSwitcherChange(DIAG_CONSTR_SWITCHER, startOption);
    }

    private createInnerLayout() {
        // create scrollable outter container
        this.scrollViewer = new GUI.ScrollViewer(DIAG_CONSTR_CONTENT_OUTTER);
        this.scrollViewer.widthInPixels = 400;
        this.scrollViewer.heightInPixels = DIAG_CONSTR_CONTENT_HEIGHT_PX;
        // GuiScrollView.setDefaultStyle(this.scrollViewer, true); 
        // and stackpanel inside it
        this.innerLayout = new GUI.StackPanel(DIAG_CONSTR_CONTENT);
        this.innerLayout.heightInPixels = DIAG_CONSTR_CONTENT_HEIGHT_PX;
        const innerLayoutBgImg = new GUI.Image(DIAG_CONSTR_CONTENT_OUTTER + "img", GUI_CONST.BG_PAPER);
        innerLayoutBgImg.heightInPixels = DIAG_CONSTR_CONTENT_HEIGHT_PX;
        // add bg and inner stackpanel to bld list layout
        this.scrollViewer.addControl(innerLayoutBgImg);
        this.scrollViewer.addControl(this.innerLayout);
        // add to constr dialog
        this.addControl(this.scrollViewer, true);
    }

    private addConstructableList(bldCatNum: number, hideNonProducable?: boolean) {
        const bldCatName = DIAG_CONSTR_SWITCHER_OPTIONS_NAMES[bldCatNum];
        this.log(`showing constructable buildings of category: ${bldCatNum} "${bldCatName}"`);
        let bldTypesOfCat: IConstructable[] = globalThis.CONSTRUCTABLE.getByCategory(bldCatNum);
        this.log(`${bldTypesOfCat.length} building types in cat`);
        for (const bldType of bldTypesOfCat) {
            // when building is of production categoory & cities cant produce this resource
            let hideType: boolean = false;
            if (hideNonProducable == true && bldType.category.isProd) {
                hideType = this.prodBldTypeIsHidden(bldType.id);
            }
            // show if not hidden
            if (! hideType) {
                this.addConstructableItem(bldType);
            }
        }
    }

    private prodBldTypeIsHidden(bldId: string): boolean {
        const resId = bldId.replace('prod-', '');
        // const resId = ProductionBuildingType.ResFromTypeId(bldId).id;
        const canBuild = this.city.canProduce(resId);
        if (! canBuild) {
            this.log(`building ${bldId} cant be build here; no neighboring ${resId} tiles`);
            return true;
        }
        return false;
    }

    private addConstructableItem(bldType: IConstructable) {
        const elemId = `lbl-build-${bldType.id}`;
        const constrRow = this.addNewDataRow(bldType.name, "Costs: 0", "building", true, elemId, this.innerLayout);
        constrRow.name = elemId;
    }

    onSwitcherChange(elemId: string, bldTypeNameStr: string): void {
        super.onSwitcherChange(elemId, bldTypeNameStr);
        if (elemId == DIAG_CONSTR_SWITCHER) {
            this.innerLayout.clearControls();
            const bldCatNameIndex = DIAG_CONSTR_SWITCHER_OPTIONS_NAMES.indexOf(bldTypeNameStr);
            const bldCatNum = DIAG_CONSTR_SWITCHER_OPTIONS_ID[bldCatNameIndex]
            this.addConstructableList(bldCatNum, true);
        }
    }
}

/* == CONSTANTS == */
export const DIAG_CONSTR = "diag-constr";
export const DIAG_CONSTR_BTN_BUILD = DIAG_CONSTR + "-btn-build";
export const DIAG_CONSTR_SWITCHER = DIAG_CONSTR + "-bld-cat";
export const DIAG_CONSTR_SWITCHER_OPTIONS_NAMES = ["Civic", "Resources", "Military"];
export const DIAG_CONSTR_SWITCHER_OPTIONS_ID = [ BuildingCategory.CIVIC, BuildingCategory.PROD, BuildingCategory.MIL ];
export const DIAG_CONSTR_CONTENT = DIAG_CONSTR + "-content";
export const DIAG_CONSTR_CONTENT_OUTTER = DIAG_CONSTR_CONTENT + "-bg";
export const DIAG_CONSTR_CONTENT_HEIGHT_PX = 400;
export const DIAG_CONSTR_BLD_PREFIX = DIAG_CONSTR + "-bld-";