// entities
import { City } from "../../entities/city/city";
import { SinglePopulationClass } from "../../entities/city/population-class";
// gui
import { EmptyDialog, IDialog } from "./common/base-dialog";
import { ColorPalette } from "../../../helpers/colorpalette";

/* == CITY CONSUMPTION DIAG MODEL == */
export class CityConsumptionDialog extends EmptyDialog {
    city: City;
    consumedResIds: Array<string>;

    constructor(city: City) {
        super(DIAG_CONSUMPTION, false, `${city.name} Consumption`);
        // set city & classes list
        this.city = city;
        this.consumedResIds = city.population.getTotalResConsumptionList();
    }

    addCustomContent(): void {
        this.log(`consumed resources: ${this.consumedResIds}`)
        for (const resId of this.consumedResIds) {
            const consumedResAmount = this.city.population.getTotalResConsumption(resId);
            if (consumedResAmount > 0) {
                const resName = globalThis.TRADE.getResName(resId);
                const datarow = this.addNewDataRow(resName, consumedResAmount.toFixed(2), "storage");
                // text color
                if (! this.city.population.resIsFullySupplied(resId)) {
                    datarow.valueLabel.color = ColorPalette.NameAsHexStr(TEXT_COLOR_RED_NAME);
                }
            } // end of consumption amount heck

        } // end of cunsumed resId loop
    }

    /* EVENT HANDLING */
    onElementClicked(elemId: string): boolean {
        const success = super.onElementClicked(elemId);
        const city = this.city;
        return false;
    }
}
/* == CONSUMPTION DIALOG CONSTANTS == */
export const DIAG_CONSUMPTION = "diag-consumption";
export const TEXT_COLOR_RED_NAME = "BROWN";
