// common
import { ColorPalette } from "../../../../helpers/colorpalette";
// gui
import * as GUI_EV from "../../common/gui-events";
import * as GUI_CONST from '../../common/constants';
import { EmptyDialog, IDialog } from "./base-dialog";
// babylon
import * as GUI from 'babylonjs-gui';
import { FlexGuiLayout } from "../../common/layouts";
import { StrProperty } from "../../../../common/properties";

/* == DIALOG BUTTON CLASS == */
export class DialogButton extends GUI.Button {
    dialog: IDialog;
    btnId: string;
    text: string;

    static CreateNew(btnId: string, text: string, parentDialog: IDialog): GUI.Button {
        // add button
        const bgImgPath = GUI_CONST.BG_BUTTON1;
        const btn = GUI.Button.CreateImageWithCenterTextButton(btnId, text, bgImgPath);
        // set default style
        DialogButton.AdjustContainer(btn);
        DialogButton.SetFontStyle(btn);
        // add on click listener
        parentDialog.addControl(btn);
        const onClickFunction = function() {
            let btnId: string = "" + btn.uniqueId;
            if (btn.name) { btnId = btn.name; }
            parentDialog.onButtonClick(btnId);
        }
        btn.onPointerDownObservable.add( onClickFunction );
        return btn;
    }

    static CreateFromProperty(strProp: StrProperty, parentDialog: IDialog): GUI.Button {
        const btnId = strProp.id;
        const btnText = strProp.value;
        return DialogButton.CreateNew(btnId, btnText, parentDialog);
    }

    static AdjustContainer(btn: GUI.Button) {
        btn.thickness = 0;
        btn.width = GUI_CONST.DEFAULT_INNER_WIDTH;
    }

    static SetFontStyle(btn: GUI.Button) {
        btn.fontFamily = GUI_CONST.FONT_HIGHLIGHTED;
        btn.fontSize = GUI_CONST.TEXTSIZE_BIGGER;
        btn.color = ColorPalette.NameAsHexStr(BUTTON_TEXT_COLOR);
    }

    
}

/* == DIALOG BUTTON CONSTANTS == */
export const BUTTON_TEXT_COLOR = "ATLANTIS";