// common
import { DawnbringerPalette } from "../../../common/colors";
// events
import { IBaseEvent } from "../../../common/events";
import * as EV from "../../game-events";
// entities
import { IFamily } from "../../entities/faction/family";
import { UnitArmy } from "../../entities/unit/army/army";
import { ArmyCategory } from "../../entities/unit/army/army-category";
// gui basics
import * as GUI_CONST from '../common/constants';
import { EmptyDialog } from "./common/base-dialog";
import { GuiRectangle } from "../common/rectangle";
// gui dialogs
import { ArmiesDialog } from "./armies-dialog";
// babylon
import * as GUI from 'babylonjs-gui';

/* == DIALOG MODEL == */
export class FamilyCaravansDialog extends EmptyDialog {
    familyNum: number = 0;

    constructor(noPlayerFamilyNum?: number) {
        super(DIAG_FAMILY_CARAVANS, false, "Caravans");
        if (noPlayerFamilyNum) { this.familyNum = noPlayerFamilyNum; }
    }

    addCustomContent(): void {
        this.addCaravanArmyList();
    }

    addCaravanArmyList() {
        for (const familyCaravan of this.familyCaravanArmies) {
            const elemId: string = "datarow-Army" + familyCaravan.num;
            this.addNewDataRow("Army", familyCaravan.name, "army", false, elemId);
        }
    }

    get isForPlayer(): boolean {
        return this.familyNum == 0;
    }

    get family(): IFamily {
        const family = globalThis.FACTIONS.getFamilyByNum(this.familyNum);
        return family;
    }

    get familyCaravanArmies(): Array<UnitArmy> {
        const factionKey = "F" + this.familyNum;
        const factionArmies = globalThis.ARMIES.getArmiesByFactionKey(factionKey);
        const factionCaravans: Array<UnitArmy> = [];
        for (const factionArmy of factionArmies) {
            if (factionArmy.isCaravan) { factionCaravans.push(factionArmy); }
        }
        return factionCaravans;
    }
}

/* == DIALOG CONSTANTS == */
export const DIAG_FAMILY_CARAVANS = "diag-caravans";