// gui
import * as GUI_CONST from '../common/constants';
// babylon
import * as BABYLON from 'babylonjs';
import * as GUI from 'babylonjs-gui';
// menues
import { AbstractMenu } from './common/abstract-menu';

export class CircularButton extends AbstractMenu {
    // basic properties
    btnId: string;
    // children
    inner: GUI.Image = null;
    // image textures
    cachedTextures: Array<BABYLON.Texture> = [];

    constructor(btnId: string, horizontalAlign: number) {
        super( CircularButton.ShortElementId(btnId) );
        this.btnId = btnId;
        // on screen position
        this.verticalAlignment = GUI_CONST.ALIGN_V_CENTER;
        this.horizontalAlignment = horizontalAlign;
        // sizing & offset
        this.setSizeInPixels(CIRCBTN_SIZE_PX)
        this.calculateOffset();
        // add inner image
        this.addInnerImage();
    }

    calculateOffset() {
        this._addYOffset();
        this._addMarginOffset();
    }

    cacheTextures() {
        
    }

    _addMarginOffset() {
        let marginXOffsetPx = GUI_CONST.BODY_MARGIN_X_PX;
        if ( this.horizontalAlignment == GUI_CONST.ALIGN_H_RIGHT) {
            marginXOffsetPx *= -1;
        }
        this.leftInPixels = marginXOffsetPx;
    }

    _addYOffset() {
        this.top = CIRCBTN_V_OFFSET;
    }

    
    addInnerImage() {
        // get name of button image in img/ui/buttons folder
        const innerImgId = CircularButton.FullElementId(this.btnId).replace("outter", "inner");
        const imageFilePath = CircularButton.ImagePath(this.btnId, false);
        this.inner = new GUI.Image(innerImgId, imageFilePath);
        // sizing
        this.inner.width = CIRCBTN_SIZE; 
        this.inner.height = CIRCBTN_SIZE;
        // add to outter
        this.addControl(this.inner);
    }

    updateImage() {
        this.inner.source = CircularButton.ImagePath(this.btnId, this.isHovered);
    }

    static ImagePath(btnId, isHovered?: boolean): string {
        let fileEnding: string = ".png";
        if (isHovered) { fileEnding = "-glowing.png" }
        return GUI_CONST.FOLDER_BUTTONS_IMG + CIRCBTN_ID_PREFIX + btnId + fileEnding;
    }

    static FullElementId(btnId: string): string {
        return CIRCBTN_ID_PREFIX_FULL + btnId + "-outter";
    }

    static ShortElementId(btnId: string): string {
        return CIRCBTN_ID_PREFIX + btnId;
    }

    onHover() {
        super.onHover();
        this.updateImage();
    }

    onHoverEnd() {
        super.onHoverEnd();
        this.updateImage();
    }

}

export const CIRCBTN_ID_PREFIX = "circbtn-";
export const CIRCBTN_ID_PREFIX_FULL = "menu-circbtn-";
export const CIRCBTN_IMG_FILE_PREFIX = "circbtn-"
export const CIRCBTN_SIZE = "120px";
export const CIRCBTN_SIZE_PX = 120;
export const CIRCBTN_V_OFFSET = "18%";
export const CIRCBTN_ICON_FOLDER = "./res/img/ui/buttons/";