// jquery
import * as $ from "jquery";
// events
import { IBaseEvent, BaseEventListener } from '../../common/events';
import * as EV from "../game-events";
// managers
import { AbstractManager } from "./abstract-manager";

export class FileManager extends AbstractManager {
    jsonContent: {} = {};
    _loadingErrors: Array<string> = [];

    constructor() {
        super("File");
    }

    loadJson(fullPath: string) {
        this.log(`start loading character names from ${fullPath}`);
        $.getJSON(fullPath)
        .done( function (data) {
            globalThis.FILES._onJsonLoaded(fullPath, data);
        })
        .fail( function() { 
            globalThis.FILES.onLoadingError(fullPath); 
        });
    }

    /* INTERNAL ON EVENT */
    _onJsonLoaded(fullPath: string, content: {}) {
        const fileNameShort = FileManager.FileNameFromPath(fullPath, true);
        this.jsonContent[fileNameShort] = content;
        this.onJsonLoaded(fileNameShort, content);
    }

    onLoadingError(fullPath: string) {
        const fileName = FileManager.FileNameFromPath(fullPath);
        this.warn(`error while loading file "${fileName}"; aborted`);
        this._loadingErrors.push(fileName);
        // fire event
        const loadingErrorEvent = new EV.FileLoadingErrorEvent(fullPath);
        globalThis.EVENTS.fireEvent(loadingErrorEvent);
    }

    onJsonLoaded(fileNameShort: string, content: {}) {
        const contentKeys = Object.keys(content).length;
        this.log(`file "${fileNameShort}" loaded with ${contentKeys} properties`);
    }

    static FileNameFromPath(fullPath: string, withoutEnding?: boolean): string {
        const splittedPath = fullPath.split("/");
        const filePath = splittedPath[splittedPath.length -1];
        if (withoutEnding) {
            const nameAndEnding = filePath.split(".");
            return nameAndEnding[0];
        }
        return filePath;
    }

    /* EVENT HANDLING */
    onEvent(ev: IBaseEvent): void {
        super.onEvent(ev);
    }

    /* GETTER METHODS */
    fullJsonPath(shortName: string) { return this.DEFAULT_FILES_PREFIX + shortName + this.ENDING_JSON; }

    /* PATH CONSTANTS */
    get DEFAULT_FILES_PREFIX(): string { return "./res/data/default/default-"; }
    get ENDING_JSON(): string { return ".json"; }
    get DEFAULT_LANG_JSON_PATH(): string { return this.fullJsonPath("languages"); }
    get DEFAULT_MOUNTS_JSON_PATH(): string { return this.fullJsonPath("unit-mounts"); }
    get DEFAULT_GEAR_JSON_PATH(): string { return this.fullJsonPath("unit-gear"); }
    get DEFAULT_BLDTYPES_JSON_PATH(): string { return this.fullJsonPath("building-types"); }
}