// common
import { Loggable } from "../../common/log";
// world
import { WorldPos } from "./pos";

/* == INTERFACES == */
export interface ILandscapeFeature {
    id: string;
    pos: WorldPos;
}

/* == MODEL == */
export class LandscapeFeature implements ILandscapeFeature {
    id: string;
    pos: WorldPos;

    constructor(id: string, x: number, z:number) {
        this.id = id;
        this.pos = new WorldPos(x, z);
    }

    static FromString(landscapeFeatureStr: string): LandscapeFeature {
        const featureData: Array<string> = landscapeFeatureStr.split(" ");
        const featureId: string = featureData[0];
        const featurePosStr: string = featureData[1] + " " + featureData[2];
        const featurePos: WorldPos = WorldPos.FromString(featurePosStr);
        const landscapeFeature = new LandscapeFeature(featureId, featurePos.x, featurePos.z);
        return landscapeFeature;
    }
}

/* == CONSTANTS == */
// ids
export const FEATURE_FOREST = "forest";
export const FEATURE_FOREST_NORTHERN = "forest-northern";
export const FEATURE_FOREST_SOUTHERN = "forest-southern";
export const FEATURE_TREASURE = "treasure";
export const FEATURE_SHIPWRECK = "shipwreck";
export const FEATURE_VOLCANO = "volcano";